@@ -1,7 +1,6 @@
 /**
- * $Id: GraphEditor.java,v 1.8 2011-02-14 15:45:58 gaudenz Exp $
- * Copyright (c) 2006-2010, Gaudenz Alder, David Benson
- */
+ * $Id: GraphEditor.java,v 1.16 2012-10-29 10:49:06 david Exp $
+ * Copyright (c) 2006-2012, JGraph Ltd */
 package com.mxgraph.examples.swing;
 
 import java.awt.Color;
@@ -26,6 +25,7 @@ import com.mxgraph.model.mxICell;
 import com.mxgraph.model.mxIGraphModel;
 import com.mxgraph.swing.mxGraphComponent;
 import com.mxgraph.swing.util.mxGraphTransferable;
+import com.mxgraph.swing.util.mxSwingConstants;
 import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxEvent;
 import com.mxgraph.util.mxEventObject;
@@ -38,7 +38,6 @@ import com.mxgraph.view.mxGraph;
 
 public class GraphEditor extends BasicGraphEditor
 {
-
 	/**
 	 * 
 	 */
@@ -433,8 +432,8 @@ public class GraphEditor extends BasicGraphEditor
 	}
 
 	/**
-	* 
-	*/
+	 * 
+	 */
 	public static class CustomGraphComponent extends mxGraphComponent
 	{
 
@@ -663,9 +662,9 @@ public class GraphEditor extends BasicGraphEditor
 			e1.printStackTrace();
 		}
 
-		mxConstants.SHADOW_COLOR = Color.LIGHT_GRAY;
+		mxSwingConstants.SHADOW_COLOR = Color.LIGHT_GRAY;
 		mxConstants.W3C_SHADOWCOLOR = "#D3D3D3";
-		
+
 		GraphEditor editor = new GraphEditor();
 		editor.createFrame(new EditorMenuBar(editor)).setVisible(true);
 	}
