@@ -1,11 +1,6 @@
 /*
- * $Id: EditorActions.java,v 1.35 2011-02-14 15:45:58 gaudenz Exp $
- * Copyright (c) 2001-2010, Gaudenz Alder, David Benson
- * 
- * All rights reserved.
- * 
- * See LICENSE file for license details. If you are unable to locate
- * this file please contact info (at) jgraph (dot) com.
+ * $Id: EditorActions.java,v 1.38 2012-09-20 14:59:30 david Exp $
+ * Copyright (c) 2001-2012, JGraph Ltd
  */
 package com.mxgraph.examples.swing.editor;
 
@@ -57,8 +52,6 @@ import com.mxgraph.canvas.mxICanvas;
 import com.mxgraph.canvas.mxSvgCanvas;
 import com.mxgraph.io.mxCodec;
 import com.mxgraph.io.mxGdCodec;
-import com.mxgraph.io.mxVdxCodec;
-import com.mxgraph.io.gd.mxGdDocument;
 import com.mxgraph.model.mxCell;
 import com.mxgraph.model.mxGraphModel;
 import com.mxgraph.model.mxIGraphModel;
@@ -71,8 +64,10 @@ import com.mxgraph.swing.view.mxCellEditor;
 import com.mxgraph.util.mxCellRenderer;
 import com.mxgraph.util.mxCellRenderer.CanvasFactory;
 import com.mxgraph.util.mxConstants;
+import com.mxgraph.util.mxDomUtils;
 import com.mxgraph.util.mxResources;
 import com.mxgraph.util.mxUtils;
+import com.mxgraph.util.mxXmlUtils;
 import com.mxgraph.util.png.mxPngEncodeParam;
 import com.mxgraph.util.png.mxPngImageEncoder;
 import com.mxgraph.util.png.mxPngTextDecoder;
@@ -559,7 +554,7 @@ public class EditorActions
 			// Creates the URL-encoded XML data
 			mxCodec codec = new mxCodec();
 			String xml = URLEncoder.encode(
-					mxUtils.getXml(codec.encode(graph.getModel())), "UTF-8");
+					mxXmlUtils.getXml(codec.encode(graph.getModel())), "UTF-8");
 			mxPngEncodeParam param = mxPngEncodeParam
 					.getDefaultEncodeParam(image);
 			param.setCompressedText(new String[] { "mxGraphModel", xml });
@@ -725,7 +720,7 @@ public class EditorActions
 													int width, int height)
 											{
 												mxSvgCanvas canvas = new mxSvgCanvas(
-														mxUtils.createSvgDocument(
+														mxDomUtils.createSvgDocument(
 																width, height));
 												canvas.setEmbedded(true);
 
@@ -734,18 +729,18 @@ public class EditorActions
 
 										});
 
-						mxUtils.writeFile(mxUtils.getXml(canvas.getDocument()),
+						mxUtils.writeFile(mxXmlUtils.getXml(canvas.getDocument()),
 								filename);
 					}
 					else if (selectedFilter == vmlFileFilter)
 					{
-						mxUtils.writeFile(mxUtils.getXml(mxCellRenderer
+						mxUtils.writeFile(mxXmlUtils.getXml(mxCellRenderer
 								.createVmlDocument(graph, null, 1, null, null)
 								.getDocumentElement()), filename);
 					}
 					else if (ext.equalsIgnoreCase("html"))
 					{
-						mxUtils.writeFile(mxUtils.getXml(mxCellRenderer
+						mxUtils.writeFile(mxXmlUtils.getXml(mxCellRenderer
 								.createHtmlDocument(graph, null, 1, null, null)
 								.getDocumentElement()), filename);
 					}
@@ -753,7 +748,7 @@ public class EditorActions
 							|| ext.equalsIgnoreCase("xml"))
 					{
 						mxCodec codec = new mxCodec();
-						String xml = mxUtils.getXml(codec.encode(graph
+						String xml = mxXmlUtils.getXml(codec.encode(graph
 								.getModel()));
 
 						mxUtils.writeFile(xml, filename);
@@ -763,8 +758,7 @@ public class EditorActions
 					}
 					else if (ext.equalsIgnoreCase("txt"))
 					{
-						String content = mxGdCodec.encode(graph)
-								.getDocumentString();
+						String content = mxGdCodec.encode(graph);
 
 						mxUtils.writeFile(content, filename);
 					}
@@ -1618,7 +1612,7 @@ public class EditorActions
 
 				if (value != null)
 				{
-					Document document = mxUtils.parseXml(URLDecoder.decode(
+					Document document = mxXmlUtils.parseXml(URLDecoder.decode(
 							value, "UTF-8"));
 					mxCodec codec = new mxCodec(document);
 					codec.decode(document.getDocumentElement(), editor
@@ -1634,38 +1628,12 @@ public class EditorActions
 					mxResources.get("imageContainsNoDiagramData"));
 		}
 
-		/**
-		 * @throws IOException 
-		 * 
-		 */
-		protected void openVdx(BasicGraphEditor editor, File file,
-				Document document)
-		{
-			mxGraph graph = editor.getGraphComponent().getGraph();
-
-			// Replaces file extension with .mxe
-			String filename = file.getName();
-			filename = filename.substring(0, filename.length() - 4) + ".mxe";
-
-			if (new File(filename).exists()
-					&& JOptionPane.showConfirmDialog(editor,
-							mxResources.get("overwriteExistingFile")) != JOptionPane.YES_OPTION)
-			{
-				return;
-			}
-
-			((mxGraphModel) graph.getModel()).clear();
-			mxVdxCodec.decode(document, graph);
-			editor.getGraphComponent().zoomAndCenter();
-			editor.setCurrentFile(new File(lastDir + "/" + filename));
-		}
-
 		/**
 		 * @throws IOException
 		 *
 		 */
 		protected void openGD(BasicGraphEditor editor, File file,
-				mxGdDocument document)
+				String gdText)
 		{
 			mxGraph graph = editor.getGraphComponent().getGraph();
 
@@ -1681,7 +1649,7 @@ public class EditorActions
 			}
 
 			((mxGraphModel) graph.getModel()).clear();
-			mxGdCodec.decode(document, graph);
+			mxGdCodec.decode(gdText, graph);
 			editor.getGraphComponent().zoomAndCenter();
 			editor.setCurrentFile(new File(lastDir + "/" + filename));
 		}
@@ -1761,35 +1729,24 @@ public class EditorActions
 								else if (fc.getSelectedFile().getAbsolutePath()
 										.toLowerCase().endsWith(".txt"))
 								{
-									mxGdDocument document = new mxGdDocument();
-									document.parse(mxUtils.readFile(fc
-											.getSelectedFile()
-											.getAbsolutePath()));
 									openGD(editor, fc.getSelectedFile(),
-											document);
+											mxUtils.readFile(fc
+													.getSelectedFile()
+													.getAbsolutePath()));
 								}
 								else
 								{
-									Document document = mxUtils
+									Document document = mxXmlUtils
 											.parseXml(mxUtils.readFile(fc
 													.getSelectedFile()
 													.getAbsolutePath()));
 
-									if (fc.getSelectedFile().getAbsolutePath()
-											.toLowerCase().endsWith(".vdx"))
-									{
-										openVdx(editor, fc.getSelectedFile(),
-												document);
-									}
-									else
-									{
-										mxCodec codec = new mxCodec(document);
-										codec.decode(
-												document.getDocumentElement(),
-												graph.getModel());
-										editor.setCurrentFile(fc
-												.getSelectedFile());
-									}
+									mxCodec codec = new mxCodec(document);
+									codec.decode(
+											document.getDocumentElement(),
+											graph.getModel());
+									editor.setCurrentFile(fc
+											.getSelectedFile());
 
 									resetEditor(editor);
 								}
