@@ -1,6 +1,6 @@
 /**
- * $Id: mxGraphics2DCanvas.java,v 1.195 2011-09-28 11:31:50 gaudenz Exp $
- * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
+ * $Id: mxGraphics2DCanvas.java,v 1.201 2012-05-30 15:39:34 gaudenz Exp $
+ * Copyright (c) 2007-2012, JGraph Ltd
  */
 package com.mxgraph.canvas;
 
@@ -40,8 +40,10 @@ import com.mxgraph.shape.mxLabelShape;
 import com.mxgraph.shape.mxLineShape;
 import com.mxgraph.shape.mxRectangleShape;
 import com.mxgraph.shape.mxRhombusShape;
+import com.mxgraph.shape.mxStencilRegistry;
 import com.mxgraph.shape.mxSwimlaneShape;
 import com.mxgraph.shape.mxTriangleShape;
+import com.mxgraph.swing.util.mxSwingConstants;
 import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxPoint;
 import com.mxgraph.util.mxRectangle;
@@ -154,8 +156,14 @@ public class mxGraphics2DCanvas extends mxBasicCanvas
 	public mxIShape getShape(Map<String, Object> style)
 	{
 		String name = mxUtils.getString(style, mxConstants.STYLE_SHAPE, null);
+		mxIShape shape = shapes.get(name);
 
-		return shapes.get(name);
+		if (shape == null && name != null)
+		{
+			shape = mxStencilRegistry.getStencil(name);
+		}
+
+		return shape;
 	}
 
 	/**
@@ -292,11 +300,12 @@ public class mxGraphics2DCanvas extends mxBasicCanvas
 				int w, h;
 				int x = bounds.x;
 				int y = bounds.y;
-				
+				Dimension size = getImageSize(img);
+
 				if (preserveAspect)
 				{
-					Dimension size = getImageSize(img);
-					double s = Math.min(bounds.width / (double) size.width, bounds.height / (double) size.height);
+					double s = Math.min(bounds.width / (double) size.width,
+							bounds.height / (double) size.height);
 					w = (int) (size.width * s);
 					h = (int) (size.height * s);
 					x += (bounds.width - w) / 2;
@@ -307,8 +316,9 @@ public class mxGraphics2DCanvas extends mxBasicCanvas
 					w = bounds.width;
 					h = bounds.height;
 				}
-				
-				Image scaledImage = img.getScaledInstance(w, h, IMAGE_SCALING);
+
+				Image scaledImage = (w == size.width && h == size.height) ? img
+						: img.getScaledInstance(w, h, IMAGE_SCALING);
 
 				if (scaledImage != null)
 				{
@@ -321,25 +331,25 @@ public class mxGraphics2DCanvas extends mxBasicCanvas
 						int sy = 1;
 						int dx = 0;
 						int dy = 0;
-						
+
 						if (flipH)
 						{
 							sx = -1;
 							dx = -w - 2 * x;
 						}
-						
+
 						if (flipV)
 						{
 							sy = -1;
 							dy = -h - 2 * y;
 						}
-						
+
 						g.scale(sx, sy);
 						g.translate(dx, dy);
 					}
 
 					drawImageImpl(scaledImage, x, y);
-					
+
 					// Restores the previous transform
 					if (af != null)
 					{
@@ -365,7 +375,7 @@ public class mxGraphics2DCanvas extends mxBasicCanvas
 	{
 		return new Dimension(image.getWidth(null), image.getHeight(null));
 	}
-	
+
 	/**
 	 * 
 	 */
@@ -471,7 +481,7 @@ public class mxGraphics2DCanvas extends mxBasicCanvas
 			// Saves the state and configures the graphics object
 			Paint p = g.getPaint();
 			Color previousColor = g.getColor();
-			g.setColor(mxConstants.SHADOW_COLOR);
+			g.setColor(mxSwingConstants.SHADOW_COLOR);
 			g.translate(shadowOffsetX, shadowOffsetY);
 
 			// Paints the shadow
@@ -498,12 +508,12 @@ public class mxGraphics2DCanvas extends mxBasicCanvas
 		{
 			float[] dashPattern = mxUtils.getFloatArray(style,
 					mxConstants.STYLE_DASH_PATTERN,
-					mxConstants.DEFAULT_DASHED_PATTERN);
+					mxConstants.DEFAULT_DASHED_PATTERN, " ");
 			float[] scaledDashPattern = new float[dashPattern.length];
 
 			for (int i = 0; i < dashPattern.length; i++)
 			{
-				scaledDashPattern[i] = (float) (dashPattern[i] * scale);
+				scaledDashPattern[i] = (float) (dashPattern[i] * scale * width);
 			}
 
 			return new BasicStroke((float) width, BasicStroke.CAP_BUTT,
