@@ -13,14 +13,22 @@ import java.awt.Paint;
 import java.awt.Rectangle;
 import java.awt.RenderingHints;
 import java.awt.Stroke;
-import java.awt.geom.Arc2D;
+import java.awt.font.TextAttribute;
+import java.awt.geom.AffineTransform;
 import java.awt.geom.Ellipse2D;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
+import java.text.AttributedString;
+import java.util.LinkedHashMap;
+import java.util.Map;
 import java.util.Stack;
 
+import javax.swing.CellRendererPane;
+import javax.swing.JLabel;
+
 import com.mxgraph.util.mxConstants;
+import com.mxgraph.util.mxLightweightLabel;
 import com.mxgraph.util.mxUtils;
 
 /**
@@ -35,6 +43,10 @@ import com.mxgraph.util.mxUtils;
  * reader.setContentHandler(new mxSaxOutputHandler(new mxGraphicsCanvas2D(g2)));
  * reader.parse(new InputSource(new StringReader(xml)));
  * </code>
+ * 
+ * Text rendering is available for plain text and HTML markup, the latter with optional
+ * word wrapping. CSS support is limited to the following:
+ * http://docs.oracle.com/javase/6/docs/api/index.html?javax/swing/text/html/CSS.html
  */
 public class mxGraphicsCanvas2D implements mxICanvas2D
 {
@@ -45,6 +57,11 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 	 */
 	public static int IMAGE_SCALING = Image.SCALE_SMOOTH;
 
+	/**
+	 * Specifies the size of the cache used to store parsed colors
+	 */
+	public static int COLOR_CACHE_SIZE = 100;
+
 	/**
 	 * Reference to the graphics instance for painting.
 	 */
@@ -106,6 +123,22 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 	 */
 	protected transient Color currentShadowColor;
 
+	/**
+	 * Optional renderer pane to be used for HTML label rendering.
+	 */
+	protected CellRendererPane rendererPane;
+
+	/**
+	 * Caches parsed colors.
+	 */
+	@SuppressWarnings("serial")
+	protected transient LinkedHashMap<String, Color> colorCache = new LinkedHashMap<String, Color>() {
+        @Override
+        protected boolean removeEldestEntry(Map.Entry<String, Color> eldest) {
+                return size() > COLOR_CACHE_SIZE;
+        }
+	};
+
 	/**
 	 * Constructs a new graphics export canvas.
 	 */
@@ -113,6 +146,16 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 	{
 		setGraphics(g);
 		state.g = g;
+
+		// Initializes the cell renderer pane for drawing HTML markup
+		try
+		{
+			rendererPane = new CellRendererPane();
+		}
+		catch (Exception e)
+		{
+			// ignore
+		}
 	}
 
 	/**
@@ -193,7 +236,7 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 	{
 		// This implementation uses custom scale/translate and built-in rotation
 		state.scale = state.scale * value;
-		currentFont = null;
+		state.strokeWidth *= value;
 	}
 
 	/**
@@ -212,12 +255,14 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 	public void rotate(double theta, boolean flipH, boolean flipV, double cx,
 			double cy)
 	{
-		cx += state.dx;
-		cy += state.dy;
-
 		cx *= state.scale;
 		cy *= state.scale;
 
+		// This is a special case where the rotation center is scaled so dx/dy,
+		// which are also scaled, must be applied after scaling the center.
+		cx += state.dx;
+		cy += state.dy;
+		
 		// This implementation uses custom scale/translate and built-in rotation
 		// Rotation state is part of the AffineTransform in state.transform
 		if (flipH ^ flipV)
@@ -284,20 +329,14 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 	 */
 	public void setDashPattern(String value)
 	{
-		// FIXME: Initial dash pattern (3, 3) isn't scaled
-		if (!state.dashPattern.equals(value))
+		if (value != null && !state.dashPattern.equals(value) && value.length() > 0)
 		{
-			float[] dashpattern = null;
+			String[] tokens = value.split(" ");
+			float[] dashpattern = new float[tokens.length];
 
-			if (state.dashed && state.dashPattern != null)
+			for (int i = 0; i < tokens.length; i++)
 			{
-				String[] tokens = value.split(" ");
-				dashpattern = new float[tokens.length];
-
-				for (int i = 0; i < tokens.length; i++)
-				{
-					dashpattern[i] = (float) (Float.parseFloat(tokens[i]) * state.scale);
-				}
+				dashpattern[i] = (float) (Float.parseFloat(tokens[i]));
 			}
 
 			state.dashPattern = dashpattern;
@@ -454,12 +493,19 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 	}
 
 	/**
-	 * Helper method that uses {@link mxUtils#parseColor(String)}. Subclassers
-	 * can override this to implement caching for frequently used colors.
+	 * Helper method that uses {@link mxUtils#parseColor(String)}.
 	 */
 	protected Color parseColor(String hex)
 	{
-		return mxUtils.parseColor(hex);
+		Color result = colorCache.get(hex);
+		
+		if (result == null)
+		{
+			result = mxUtils.parseColor(hex);
+			colorCache.put(hex, result);
+		}
+		
+		return result;
 	}
 
 	/**
@@ -605,7 +651,16 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 	 */
 	protected Image scaleImage(Image img, int w, int h)
 	{
-		return img.getScaledInstance(w, h, IMAGE_SCALING);
+		Dimension size = getImageSize(img);
+
+		if (w == size.width && h == size.height)
+		{
+			return img;
+		}
+		else
+		{
+			return img.getScaledInstance(w, h, IMAGE_SCALING);
+		}
 	}
 
 	/**
@@ -643,50 +698,146 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 		return g2;
 	}
 
+	/**
+	 * Creates a HTML document around the given markup.
+	 */
+	protected String createHtmlDocument(String text, String align,
+			String valign, int w, int h)
+	{
+		StringBuffer css = new StringBuffer();
+		css.append("font-family:" + state.fontFamily + ";");
+		css.append("font-size:" + Math.floor(state.fontSize / state.scale) + " pt;");
+		css.append("color:" + state.fontColorValue + ";");
+
+		if ((state.fontStyle & mxConstants.FONT_BOLD) == mxConstants.FONT_BOLD)
+		{
+			css.append("font-weight:bold;");
+		}
+
+		if ((state.fontStyle & mxConstants.FONT_ITALIC) == mxConstants.FONT_ITALIC)
+		{
+			css.append("font-style:italic;");
+		}
+
+		if ((state.fontStyle & mxConstants.FONT_UNDERLINE) == mxConstants.FONT_UNDERLINE)
+		{
+			css.append("text-decoration:underline;");
+		}
+
+		if (align != null)
+		{
+			if (align.equals(mxConstants.ALIGN_CENTER))
+			{
+				css.append("text-align:center;");
+			}
+			else if (align.equals(mxConstants.ALIGN_RIGHT))
+			{
+				css.append("text-align:right;");
+			}
+		}
+
+		return "<html><div style=\"width:" + w + "px;height:" + h + "px;"
+				+ css.toString() + "\">" + text + "</div></html>";
+	}
+	
+	/**
+	 * Hook to return the renderer for HTML formatted text. This implementation returns
+	 * the shared instance of mxLighweightLabel.
+	 */
+	protected JLabel getTextRenderer()
+	{
+		return mxLightweightLabel.getSharedInstance();
+	}
+
 	/**
 	 * Draws the given text.
 	 */
 	public void text(double x, double y, double w, double h, String str,
-			String align, String valign, boolean vertical)
+			String align, String valign, boolean vertical, boolean wrap,
+			String format)
 	{
 		if (!state.fontColorValue.equals(mxConstants.NONE))
 		{
-			x = state.dx + x * state.scale;
-			y = state.dy + y * state.scale;
-			w *= state.scale;
-			h *= state.scale;
-
-			// Font-metrics needed below this line
-			Graphics2D g2 = createTextGraphics(x, y, w, h, vertical);
-			FontMetrics fm = g2.getFontMetrics();
-			String[] lines = str.split("\n");
-
-			y = getVerticalTextPosition(x, y, w, h, align, valign, vertical,
-					fm, lines);
-			x = getHorizontalTextPosition(x, y, w, h, align, valign, vertical,
-					fm, lines);
-
-			for (int i = 0; i < lines.length; i++)
+			if (format != null && format.equals("html"))
 			{
-				double dx = 0;
+				x += state.dx / state.scale;
+				y += state.dy / state.scale;
+				
+				JLabel textRenderer = getTextRenderer();
 
-				if (align != null)
+				if (textRenderer != null && rendererPane != null)
 				{
-					if (align.equals(mxConstants.ALIGN_CENTER))
-					{
-						int sw = fm.stringWidth(lines[i]);
-						dx = (w - sw) / 2;
-					}
-					else if (align.equals(mxConstants.ALIGN_RIGHT))
-					{
-						int sw = fm.stringWidth(lines[i]);
-						dx = w - sw;
-					}
+					// Use native scaling for HTML
+					AffineTransform previous = state.g.getTransform();
+					state.g.scale(state.scale, state.scale);
+					
+					// Renders the scaled text with a correction factor of
+					// PX_PER_PIXEL for px in HTML vs pixels in the bitmap
+					str = createHtmlDocument(str, align, valign,
+							(int) Math.round(w * mxConstants.PX_PER_PIXEL),
+							(int) Math.round(h * mxConstants.PX_PER_PIXEL));
+					textRenderer.setText(str);
+					rendererPane.paintComponent(state.g, textRenderer,
+							rendererPane, (int) Math.round(x),
+							(int) Math.round(y), (int) Math.round(w),
+							(int) Math.round(h), true);
+					state.g.setTransform(previous);
 				}
+			}
+			else
+			{
+				x = state.dx + x * state.scale;
+				y = state.dy + y * state.scale;
+				w *= state.scale;
+				h *= state.scale;
 
-				g2.drawString(lines[i], (int) Math.round(x + dx),
-						(int) Math.round(y));
-				y += fm.getHeight() + mxConstants.LINESPACING;
+				// Font-metrics needed below this line
+				Graphics2D g2 = createTextGraphics(x, y, w, h, vertical);
+				FontMetrics fm = g2.getFontMetrics();
+				String[] lines = str.split("\n");
+
+				y = getVerticalTextPosition(x, y, w, h, align, valign,
+						vertical, fm, lines);
+				x = getHorizontalTextPosition(x, y, w, h, align, valign,
+						vertical, fm, lines);
+
+				for (int i = 0; i < lines.length; i++)
+				{
+					double dx = 0;
+
+					if (align != null)
+					{
+						if (align.equals(mxConstants.ALIGN_CENTER))
+						{
+							int sw = fm.stringWidth(lines[i]);
+							dx = (w - sw) / 2;
+						}
+						else if (align.equals(mxConstants.ALIGN_RIGHT))
+						{
+							int sw = fm.stringWidth(lines[i]);
+							dx = w - sw;
+						}
+					}
+
+					// Adds support for underlined text via attributed character iterator
+					if ((state.fontStyle & mxConstants.FONT_UNDERLINE) == mxConstants.FONT_UNDERLINE)
+					{
+						AttributedString as = new AttributedString(lines[i]);
+						as.addAttribute(TextAttribute.FONT, g2.getFont());
+						as.addAttribute(TextAttribute.UNDERLINE,
+								TextAttribute.UNDERLINE_ON);
+
+						g2.drawString(as.getIterator(),
+								(int) Math.round(x + dx), (int) Math.round(y));
+					}
+					else
+					{
+						g2.drawString(lines[i], (int) Math.round(x + dx),
+								(int) Math.round(y));
+					}
+
+					y += fm.getHeight() + mxConstants.LINESPACING;
+				}
 			}
 		}
 	}
@@ -843,119 +994,6 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 		}
 	}
 
-	/**
-	 * 
-	 */
-	public void arcTo(double rx, double ry, double xAxisRotation,
-			boolean largeArc, boolean sweep, double x, double y)
-	{
-		if (currentPath != null)
-		{
-			rx *= state.scale;
-			ry *= state.scale;
-
-			x = x * state.scale + state.dx;
-			y = y * state.scale + state.dy;
-
-			Point2D currentPoint = currentPath.getCurrentPoint();
-			double x0 = currentPoint.getX();
-			double y0 = currentPoint.getY();
-
-			// Compute the half distance between the current and the final point
-			double dx2 = (x0 - x) / 2.0;
-			double dy2 = (y0 - y) / 2.0;
-			// Convert angle from degrees to radians
-			xAxisRotation = Math.toRadians(xAxisRotation % 360.0);
-			double cosAngle = Math.cos(xAxisRotation);
-			double sinAngle = Math.sin(xAxisRotation);
-
-			//
-			// Step 1 : Compute (x1, y1)
-			//
-			double x1 = (cosAngle * dx2 + sinAngle * dy2);
-			double y1 = (-sinAngle * dx2 + cosAngle * dy2);
-			// Ensure radii are large enough
-			rx = Math.abs(rx);
-			ry = Math.abs(ry);
-			double Prx = rx * rx;
-			double Pry = ry * ry;
-			double Px1 = x1 * x1;
-			double Py1 = y1 * y1;
-			// check that radii are large enough
-			double radiiCheck = Px1 / Prx + Py1 / Pry;
-
-			if (radiiCheck > 1)
-			{
-				rx = Math.sqrt(radiiCheck) * rx;
-				ry = Math.sqrt(radiiCheck) * ry;
-				Prx = rx * rx;
-				Pry = ry * ry;
-			}
-
-			//
-			// Step 2 : Compute (cx1, cy1)
-			//
-			double sign = (largeArc == sweep) ? -1 : 1;
-			double sq = ((Prx * Pry) - (Prx * Py1) - (Pry * Px1))
-					/ ((Prx * Py1) + (Pry * Px1));
-			sq = (sq < 0) ? 0 : sq;
-			double coef = (sign * Math.sqrt(sq));
-			double cx1 = coef * ((rx * y1) / ry);
-			double cy1 = coef * -((ry * x1) / rx);
-
-			//
-			// Step 3 : Compute (cx, cy) from (cx1, cy1)
-			//
-			double sx2 = (x0 + x) / 2.0;
-			double sy2 = (y0 + y) / 2.0;
-			double cx = sx2 + (cosAngle * cx1 - sinAngle * cy1);
-			double cy = sy2 + (sinAngle * cx1 + cosAngle * cy1);
-
-			//
-			// Step 4 : Compute the angleStart (angle1) and the angleExtent (dangle)
-			//
-			double ux = (x1 - cx1) / rx;
-			double uy = (y1 - cy1) / ry;
-			double vx = (-x1 - cx1) / rx;
-			double vy = (-y1 - cy1) / ry;
-			double p, n;
-			// Compute the angle start
-			n = Math.sqrt((ux * ux) + (uy * uy));
-			p = ux; // (1 * ux) + (0 * uy)
-			sign = (uy < 0) ? -1.0 : 1.0;
-			double angleStart = Math.toDegrees(sign * Math.acos(p / n));
-
-			// Compute the angle extent
-			n = Math.sqrt((ux * ux + uy * uy) * (vx * vx + vy * vy));
-			p = ux * vx + uy * vy;
-			sign = (ux * vy - uy * vx < 0) ? -1.0 : 1.0;
-			double angleExtent = Math.toDegrees(sign * Math.acos(p / n));
-
-			if (!sweep && angleExtent > 0)
-			{
-				angleExtent -= 360f;
-			}
-			else if (sweep && angleExtent < 0)
-			{
-				angleExtent += 360f;
-			}
-
-			angleExtent %= 360f;
-			angleStart %= 360f;
-
-			Arc2D.Double arc = new Arc2D.Double();
-			arc.x = cx - rx;
-			arc.y = cy - ry;
-			arc.width = rx * 2.0;
-			arc.height = ry * 2.0;
-			arc.start = -angleStart;
-			arc.extent = -angleExtent;
-
-			currentPath.append(arc, true);
-			currentPathIsOrthogonal = false;
-		}
-	}
-
 	/**
 	 * Closes the current path.
 	 */
@@ -1042,7 +1080,7 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 	/**
 	 * 
 	 */
-	public void shadow(String value)
+	public void shadow(String value, boolean filled)
 	{
 		if (value != null && currentPath != null)
 		{
@@ -1055,7 +1093,12 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 
 			updateStroke();
 			state.g.setColor(currentShadowColor);
-			state.g.fill(currentPath);
+			
+			if (filled)
+			{
+				state.g.fill(currentPath);
+			}
+			
 			state.g.draw(currentPath);
 		}
 	}
@@ -1079,7 +1122,7 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 		// LATER: Make currentFont part of state
 		if (currentFont == null)
 		{
-			int size = (int) Math.round(state.fontSize * state.scale);
+			int size = (int) Math.floor(state.fontSize);
 
 			int style = ((state.fontStyle & mxConstants.FONT_BOLD) == mxConstants.FONT_BOLD) ? Font.BOLD
 					: Font.PLAIN;
@@ -1129,10 +1172,20 @@ public class mxGraphicsCanvas2D implements mxICanvas2D
 			}
 
 			float miterlimit = (float) state.miterLimit;
+			float[] dash = null;
+			
+			if (state.dashed)
+			{
+				dash = new float[state.dashPattern.length];
+				
+				for (int i = 0; i < dash.length; i++)
+				{
+					dash[i] = (float) (state.dashPattern[i] * state.strokeWidth);
+				}
+			}
 
 			currentStroke = new BasicStroke((float) state.strokeWidth, cap,
-					join, miterlimit,
-					(state.dashed) ? state.dashPattern : null, 0);
+					join, miterlimit, dash, 0);
 			state.g.setStroke(currentStroke);
 		}
 	}
