@@ -1,5 +1,5 @@
 /**
- * $Id: mxCodecRegistry.java,v 1.22 2010-08-02 12:24:02 david Exp $
+ * $Id: mxCodecRegistry.java,v 1.24 2010-09-15 11:57:07 gaudenz Exp $
  * Copyright (c) 2007, Gaudenz Alder
  */
 package com.mxgraph.io;
@@ -10,6 +10,12 @@ import java.util.Hashtable;
 import java.util.List;
 import java.util.Map;
 
+import com.mxgraph.model.mxGraphModel.mxCollapseChange;
+import com.mxgraph.model.mxGraphModel.mxGeometryChange;
+import com.mxgraph.model.mxGraphModel.mxStyleChange;
+import com.mxgraph.model.mxGraphModel.mxValueChange;
+import com.mxgraph.model.mxGraphModel.mxVisibleChange;
+
 /**
  * Singleton class that acts as a global registry for codecs. See
  * {@link mxCodec} for an example.
@@ -47,6 +53,15 @@ public class mxCodecRegistry
 		register(new mxModelCodec());
 		register(new mxCellCodec());
 		register(new mxStylesheetCodec());
+
+		register(new mxRootChangeCodec());
+		register(new mxChildChangeCodec());
+		register(new mxTerminalChangeCodec());
+		register(new mxGenericChangeCodec(new mxValueChange(), "value"));
+		register(new mxGenericChangeCodec(new mxStyleChange(), "style"));
+		register(new mxGenericChangeCodec(new mxGeometryChange(), "geometry"));
+		register(new mxGenericChangeCodec(new mxCollapseChange(), "collapsed"));
+		register(new mxGenericChangeCodec(new mxVisibleChange(), "visible"));
 	}
 
 	/**
@@ -60,9 +75,9 @@ public class mxCodecRegistry
 		{
 			String name = codec.getName();
 			codecs.put(name, codec);
-			
+
 			String classname = getName(codec.getTemplate());
-			
+
 			if (!classname.equals(name))
 			{
 				addAlias(classname, name);
@@ -89,12 +104,12 @@ public class mxCodecRegistry
 	public static mxObjectCodec getCodec(String name)
 	{
 		String tmp = aliases.get(name);
-		
+
 		if (tmp != null)
 		{
 			name = tmp;
 		}
-		
+
 		mxObjectCodec codec = codecs.get(name);
 
 		// Registers a new default codec for the given name
@@ -211,7 +226,7 @@ public class mxCodecRegistry
 	public static String getName(Object instance)
 	{
 		Class<? extends Object> type = instance.getClass();
-		
+
 		if (type.isArray() || Collection.class.isAssignableFrom(type)
 				|| Map.class.isAssignableFrom(type))
 		{
