@@ -1,6 +1,6 @@
 /**
- * $Id: mxCoordinateAssignment.java,v 1.11 2011-10-21 21:05:13 david Exp $
- * Copyright (c) 2005-2010, David Benson, Gaudenz Alder
+ * $Id: mxCoordinateAssignment.java,v 1.17 2012-06-21 14:20:00 david Exp $
+ * Copyright (c) 2005-2012, JGraph Ltd
  */
 
 package com.mxgraph.layout.hierarchical.stage;
@@ -30,6 +30,7 @@ import com.mxgraph.layout.hierarchical.model.mxGraphHierarchyNode;
 import com.mxgraph.layout.hierarchical.model.mxGraphHierarchyRank;
 import com.mxgraph.util.mxPoint;
 import com.mxgraph.util.mxRectangle;
+import com.mxgraph.util.mxUtils;
 import com.mxgraph.view.mxGraph;
 
 /**
@@ -180,6 +181,11 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 	 */
 	protected mxGraphAbstractHierarchyCell[][] nextLayerConnectedCache;
 
+	/**
+	 * Padding added to resized parents
+	 */
+	protected int groupPadding = 10;
+
 	/**
 	 * A store of connections to the layer below for speed
 	 */
@@ -214,6 +220,34 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 		setLoggerLevel(Level.OFF);
 	}
 
+	/**
+	 * Utility method to display the x co-ords
+	 */
+	public void printStatus()
+	{
+		mxGraphHierarchyModel model = layout.getModel();
+
+		System.out.println("======Coord assignment debug=======");
+
+		for (int j = 0; j < model.ranks.size(); j++)
+		{
+			System.out.print("Rank " + j + " : " );
+			mxGraphHierarchyRank rank = model.ranks
+					.get(new Integer(j));
+			Iterator<mxGraphAbstractHierarchyCell> iter = rank
+					.iterator();
+
+			while (iter.hasNext())
+			{
+				mxGraphAbstractHierarchyCell cell = iter.next();
+				System.out.print(cell.getX(j) + "  ");
+			}
+			System.out.println();
+		}
+		
+		System.out.println("====================================");
+	}
+	
 	/**
 	 * A basic horizontal coordinate assignment algorithm
 	 */
@@ -1067,7 +1101,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 					if (currentX == nextX)
 					{
-						upXPositions[i - cell.minRank - 1] = cell.getX(i);
+						upXPositions[i - cell.minRank - 1] = currentX;
 						upSegCount++;
 					}
 					else if (repositionValid(model, cell, i + 1, currentX))
@@ -1078,7 +1112,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 					}
 					else
 					{
-						upXPositions[i - cell.minRank - 1] = cell.getX(i);
+						upXPositions[i - cell.minRank - 1] = nextX;
 						currentX = nextX;
 					}
 				}
@@ -1093,7 +1127,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 
 					if (currentX == nextX)
 					{
-						downXPositions[i - cell.minRank - 2] = cell.getX(i);
+						downXPositions[i - cell.minRank - 2] = currentX;
 						downSegCount++;
 					}
 					else if (repositionValid(model, cell, i - 1, currentX))
@@ -1104,7 +1138,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 					}
 					else
 					{
-						downXPositions[i - cell.minRank - 2] = cell.getX(i);
+						downXPositions[i - cell.minRank - 2] = cell.getX(i-1);
 						currentX = nextX;
 					}
 				}
@@ -1115,7 +1149,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 					continue;
 				}
 
-				if (downSegCount > upSegCount)
+				if (downSegCount >= upSegCount)
 				{
 					// Apply down calculation values
 					for (int i = cell.maxRank - 2; i > cell.minRank; i--)
@@ -1146,7 +1180,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 	/**
 	 * Determines whether or not a node may be moved to the specified x 
 	 * position on the specified rank
-	 * @model the layout model
+	 * @param model the layout model
 	 * @param cell the cell being analysed
 	 * @param rank the layer of the cell
 	 * @param position the x position being sought
@@ -1246,6 +1280,13 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 			rankBottomY[i] = 0.0;
 		}
 
+		Set<Object> parentsChanged = null;
+		
+		if (layout.isResizeParent())
+		{
+			parentsChanged = new HashSet<Object>();
+		}
+		
 		Map<Object, mxGraphHierarchyEdge> edges = model.getEdgeMapper();
 		Map<Object, mxGraphHierarchyNode> vertices = model.getVertexMapper();
 
@@ -1253,9 +1294,19 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 		// limits of each rank. Between these limits lie the channels
 		// where the edges can be routed across the graph
 
-		for (mxGraphAbstractHierarchyCell cell : vertices.values())
+		for (mxGraphHierarchyNode cell : vertices.values())
 		{
 			setVertexLocation(cell);
+			
+			if (layout.isResizeParent())
+			{
+				parentsChanged.add(graph.getModel().getParent(cell.cell));
+			}
+		}
+		
+		if (layout.isResizeParent())
+		{
+			adjustParents(parentsChanged);
 		}
 
 		// Post process edge styles. Needs the vertex locations set for initial
@@ -1272,6 +1323,16 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 		}
 	}
 
+	/**
+	 * Adjust parent cells whose child geometries have changed. The default 
+	 * implementation adjusts the group to just fit around the children with 
+	 * a padding.
+	 */
+	protected void adjustParents(Set<Object> parentsChanged)
+	{
+		layout.arrangeGroups(mxUtils.sortCells(parentsChanged, true).toArray(), groupPadding);
+	}
+
 	/**
 	 * Separates the x position of edges as they connect to vertices
 	 * 
@@ -1475,22 +1536,44 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 			Iterator<Object> parallelEdges = edge.edges.iterator();
 			int parallelEdgeCount = 0;
 			double[] jettys = jettyPositions.get(edge);
+			
+			Object source = edge.isReversed() ? edge.target.cell : edge.source.cell;
 
 			while (parallelEdges.hasNext())
 			{
 				Object realEdge = parallelEdges.next();
+				Object realSource = layout.getGraph().getView().getVisibleTerminal(realEdge, true);
+				
 				List<mxPoint> newPoints = new ArrayList<mxPoint>(edge.x.length);
 
+				// Single length reversed edges end up with the jettys in the wrong
+				// places. Since single length edges only have jettys, not segment
+				// control points, we just say the edge isn't reversed in this section
+				boolean reversed = edge.isReversed();
+				
+				if (realSource != source)
+				{
+					// The real edges include all core model edges and these can go
+					// in both directions. If the source of the hierarchical model edge
+					// isn't the source of the specific real edge in this iteration
+					// treat if as reversed
+					reversed = !reversed;
+				}
+				
+				// First jetty of edge
 				if (jettys != null)
 				{
-					int arrayOffset = edge.isReversed() ? 2 : 0;
-					double y = edge.isReversed() ? rankTopY[minRank] : rankBottomY[maxRank];
+					int arrayOffset = reversed ? 2 : 0;
+					double y = reversed ? rankTopY[minRank] : rankBottomY[maxRank];
 					double jetty = jettys[parallelEdgeCount * 4 + 1 + arrayOffset];
 					
-					if (edge.isReversed())
+					// If the edge is reversed invert the y position within the channel,
+					// unless it is a single length edge
+					if (reversed)
 					{
 						jetty = -jetty;
 					}
+					
 					y += jetty;
 					double x = jettys[parallelEdgeCount * 4 + arrayOffset];
 
@@ -1513,7 +1596,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 				int loopDelta = -1;
 				int currentRank = edge.maxRank - 1;
 
-				if (edge.isReversed())
+				if (reversed)
 				{
 					loopStart = 0;
 					loopLimit = edge.x.length;
@@ -1533,7 +1616,7 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 					double topChannelY = (rankTopY[currentRank] + rankBottomY[currentRank + 1]) / 2.0;
 					double bottomChannelY = (rankTopY[currentRank - 1] + rankBottomY[currentRank]) / 2.0;
 
-					if (edge.isReversed())
+					if (reversed)
 					{
 						double tmp = topChannelY;
 						topChannelY = bottomChannelY;
@@ -1560,13 +1643,14 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 					currentRank += loopDelta;
 				}
 
+				// Second jetty of edge
 				if (jettys != null)
 				{
-					int arrayOffset = edge.isReversed() ? 2 : 0;
-					double rankY = edge.isReversed() ? rankBottomY[maxRank] : rankTopY[minRank];
+					int arrayOffset = reversed ? 2 : 0;
+					double rankY = reversed ? rankBottomY[maxRank] : rankTopY[minRank];
 					double jetty = jettys[parallelEdgeCount * 4 + 3 - arrayOffset];
 					
-					if (edge.isReversed())
+					if (reversed)
 					{
 						jetty = -jetty;
 					}
@@ -1624,6 +1708,11 @@ public class mxCoordinateAssignment implements mxHierarchicalLayoutStage
 		double positionX = node.x[0] - node.width / 2;
 		double positionY = node.y[0] - node.height / 2;
 
+//		if (cell.minRank == -1)
+//		{
+//			System.out.println("invalid rank, never set");
+//		}
+
 		rankTopY[cell.minRank] = Math.min(rankTopY[cell.minRank], positionY);
 		rankBottomY[cell.minRank] = Math.max(rankBottomY[cell.minRank],
 				positionY + node.height);
