@@ -1,5 +1,5 @@
 /**
- * $Id: mxConnectorShape.java,v 1.22 2011-05-02 13:18:22 david Exp $
+ * $Id: mxConnectorShape.java,v 1.23 2012-04-02 10:20:54 gaudenz Exp $
  * Copyright (c) 2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.shape;
@@ -147,7 +147,7 @@ public class mxConnectorShape extends mxBasicShape
 		
 		if (marker != null)
 		{
-			offset = marker.paintMarker(canvas, state, type, pe, nx, ny, absSize);
+			offset = marker.paintMarker(canvas, state, type, pe, nx, ny, absSize, source);
 			
 			if (offset != null)
 			{
