@@ -1,5 +1,5 @@
 /**
- * $Id: mxHtmlTextShape.java,v 1.5 2010-10-28 10:41:05 gaudenz Exp $
+ * $Id: mxHtmlTextShape.java,v 1.8 2012-05-10 11:29:43 gaudenz Exp $
  * Copyright (c) 2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.shape;
@@ -16,6 +16,23 @@ import com.mxgraph.util.mxLightweightLabel;
 import com.mxgraph.util.mxUtils;
 import com.mxgraph.view.mxCellState;
 
+/**
+ * To set global CSS for all HTML labels, use the following code:
+ * 
+ * <pre>
+ * mxGraphics2DCanvas.putTextShape(mxGraphics2DCanvas.TEXT_SHAPE_HTML,
+ *   new mxHtmlTextShape()
+ *   {
+ *     protected String createHtmlDocument(Map<String, Object> style, String text)
+ *     {
+ *       return mxUtils.createHtmlDocument(style, text, 1, 0,
+ *           "<style type=\"text/css\">.selectRef { " +
+ *           "font-size:9px;font-weight:normal; }</style>");
+ *     }
+ *   }
+ * );
+ * </pre> 
+ */
 public class mxHtmlTextShape implements mxITextShape
 {
 
@@ -40,6 +57,15 @@ public class mxHtmlTextShape implements mxITextShape
 	{
 		replaceHtmlLinefeeds = value;
 	}
+	
+	/**
+	 * 
+	 */
+	protected String createHtmlDocument(Map<String, Object> style,
+			String text)
+	{
+		return mxUtils.createHtmlDocument(style,  text);
+	}
 
 	/**
 	 * 
@@ -81,7 +107,7 @@ public class mxHtmlTextShape implements mxITextShape
 			}
 
 			// Renders the scaled text
-			textRenderer.setText(mxUtils.createHtmlDocument(style, text));
+			textRenderer.setText(createHtmlDocument(style, text));
 			textRenderer.setFont(mxUtils.getFont(style, canvas.getScale()));
 			g.scale(scale, scale);
 			rendererPane.paintComponent(g, textRenderer, rendererPane,
