@@ -1,5 +1,5 @@
 /**
- * $Id: mxImageShape.java,v 1.4 2010-08-11 10:33:49 david Exp $
+ * $Id: mxImageShape.java,v 1.6 2011-01-05 11:20:51 gaudenz Exp $
  * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.shape;
@@ -25,8 +25,14 @@ public class mxImageShape extends mxRectangleShape
 	{
 		super.paintShape(canvas, state);
 
-		canvas.drawImage(getImageBounds(canvas, state), getImageForStyle(
-				canvas, state));
+		boolean flipH = mxUtils.isTrue(state.getStyle(),
+				mxConstants.STYLE_IMAGE_FLIPH, false);
+		boolean flipV = mxUtils.isTrue(state.getStyle(),
+				mxConstants.STYLE_IMAGE_FLIPV, false);
+
+		canvas.drawImage(getImageBounds(canvas, state),
+				getImageForStyle(canvas, state),
+				mxGraphics2DCanvas.PRESERVE_IMAGE_ASPECT, flipH, flipV);
 	}
 
 	/**
@@ -37,6 +43,14 @@ public class mxImageShape extends mxRectangleShape
 		return state.getRectangle();
 	}
 
+	/**
+	 * 
+	 */
+	public boolean hasGradient(mxGraphics2DCanvas canvas, mxCellState state)
+	{
+		return false;
+	}
+
 	/**
 	 * 
 	 */
