@@ -1,6 +1,6 @@
 /**
- * $Id: mxGraphHandler.java,v 1.74 2011-11-04 12:46:37 gaudenz Exp $
- * Copyright (c) 2008, Gaudenz Alder
+ * $Id: mxGraphHandler.java,v 1.81 2012-07-30 11:22:37 gaudenz Exp $
+ * Copyright (c) 2008-2012, JGraph Ltd
  * 
  * Known issue: Drag image size depends on the initial position and may sometimes
  * not align with the grid when dragging. This is because the rounding of the width
@@ -40,16 +40,18 @@ import javax.swing.JComponent;
 import javax.swing.SwingUtilities;
 import javax.swing.TransferHandler;
 
+import com.mxgraph.model.mxIGraphModel;
 import com.mxgraph.swing.mxGraphComponent;
 import com.mxgraph.swing.util.mxGraphTransferable;
 import com.mxgraph.swing.util.mxMouseAdapter;
+import com.mxgraph.swing.util.mxSwingConstants;
 import com.mxgraph.util.mxCellRenderer;
-import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxEvent;
 import com.mxgraph.util.mxEventObject;
+import com.mxgraph.util.mxEventSource.mxIEventListener;
 import com.mxgraph.util.mxPoint;
 import com.mxgraph.util.mxRectangle;
-import com.mxgraph.util.mxEventSource.mxIEventListener;
+import com.mxgraph.util.mxUtils;
 import com.mxgraph.view.mxCellState;
 import com.mxgraph.view.mxGraph;
 
@@ -210,7 +212,7 @@ public class mxGraphHandler extends mxMouseAdapter implements
 	 * Workaround for alt-key-state not correct in mouseReleased. Note: State
 	 * of the alt-key is not available during drag-and-drop.
 	 */
-	private transient boolean gridEnabledEvent = false;
+	protected transient boolean gridEnabledEvent = false;
 
 	/**
 	 * Workaround for shift-key-state not correct in mouseReleased.
@@ -251,20 +253,20 @@ public class mxGraphHandler extends mxMouseAdapter implements
 
 		// Listens to dropped graph cells
 		installDropTargetHandler();
-		
+
 		// Listens to changes of the transferhandler
 		graphComponent.addPropertyChangeListener(new PropertyChangeListener()
 		{
-			
 			public void propertyChange(PropertyChangeEvent evt)
 			{
 				if (evt.getPropertyName().equals("transferHandler"))
 				{
 					if (currentDropTarget != null)
 					{
-						currentDropTarget.removeDropTargetListener(mxGraphHandler.this);
+						currentDropTarget
+								.removeDropTargetListener(mxGraphHandler.this);
 					}
-					
+
 					installDropTargetHandler();
 				}
 			}
@@ -294,7 +296,7 @@ public class mxGraphHandler extends mxMouseAdapter implements
 
 						if (t != null)
 						{
-							e.startDrag(null, mxConstants.EMPTY_IMAGE,
+							e.startDrag(null, mxSwingConstants.EMPTY_IMAGE,
 									new Point(), t, new DragSourceAdapter()
 									{
 
@@ -412,6 +414,14 @@ public class mxGraphHandler extends mxMouseAdapter implements
 	{
 		return new mxMovePreview(graphComponent);
 	}
+	
+	/**
+	 * 
+	 */
+	public mxMovePreview getMovePreview()
+	{
+		return movePreview;
+	}
 
 	/**
 	 * 
@@ -438,6 +448,7 @@ public class mxGraphHandler extends mxMouseAdapter implements
 			 */
 			public Object getCell(MouseEvent e)
 			{
+				mxIGraphModel model = graphComponent.getGraph().getModel();
 				TransferHandler th = graphComponent.getTransferHandler();
 				boolean isLocal = th instanceof mxGraphTransferHandler
 						&& ((mxGraphTransferHandler) th).isLocalDrag();
@@ -447,6 +458,20 @@ public class mxGraphHandler extends mxMouseAdapter implements
 				Object[] cells = (isLocal) ? graph.getSelectionCells()
 						: dragCells;
 				cell = graph.getDropTarget(cells, e.getPoint(), cell);
+
+				// Checks if parent is dropped into child
+				Object parent = cell;
+
+				while (parent != null)
+				{
+					if (mxUtils.contains(cells, parent))
+					{
+						return null;
+					}
+					
+					parent = model.getParent(parent);
+				}
+
 				boolean clone = graphComponent.isCloneEvent(e) && cloneEnabled;
 
 				if (isLocal && cell != null && cells.length > 0 && !clone
@@ -537,7 +562,7 @@ public class mxGraphHandler extends mxMouseAdapter implements
 	{
 		markerEnabled = value;
 	}
-	
+
 	/**
 	 * 
 	 */
@@ -545,7 +570,7 @@ public class mxGraphHandler extends mxMouseAdapter implements
 	{
 		return marker;
 	}
-	
+
 	/**
 	 * 
 	 */
@@ -978,7 +1003,7 @@ public class mxGraphHandler extends mxMouseAdapter implements
 		if (graphComponent.isAutoScroll())
 		{
 			graphComponent.getGraphControl().scrollRectToVisible(
-				new Rectangle(e.getPoint()));
+					new Rectangle(e.getPoint()));
 		}
 
 		if (!e.isConsumed())
@@ -1424,7 +1449,7 @@ public class mxGraphHandler extends mxMouseAdapter implements
 			}
 			else if (!imagePreview)
 			{
-				mxConstants.PREVIEW_BORDER.paintBorder(graphComponent, g,
+				mxSwingConstants.PREVIEW_BORDER.paintBorder(graphComponent, g,
 						previewBounds.x, previewBounds.y, previewBounds.width,
 						previewBounds.height);
 			}
