@@ -1,5 +1,5 @@
 /**
- * $Id: mxPanningHandler.java,v 1.5 2010-06-01 10:18:45 gaudenz Exp $
+ * $Id: mxPanningHandler.java,v 1.8 2010-12-08 12:31:50 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  */
 package com.mxgraph.swing.handler;
@@ -93,7 +93,7 @@ public class mxPanningHandler extends mxMouseAdapter
 			int bottom = r.y + ((dy > 0) ? 0 : r.height) - dy;
 
 			graphComponent.getGraphControl().scrollRectToVisible(
-					new Rectangle(right, bottom, 1, 1));
+					new Rectangle(right, bottom, 0, 0));
 
 			e.consume();
 		}
@@ -118,4 +118,12 @@ public class mxPanningHandler extends mxMouseAdapter
 		start = null;
 	}
 
+	/**
+	 * Whether or not panning is currently active
+	 * @return Whether or not panning is currently active
+	 */
+	public boolean isActive()
+	{
+		return (start != null);
+	}
 }
