@@ -1,5 +1,5 @@
 /**
- * $Id: mxSelectionCellsHandler.java,v 1.9 2011-09-05 08:42:15 gaudenz Exp $
+ * $Id: mxSelectionCellsHandler.java,v 1.13 2012-07-25 08:23:43 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  * 
  * Known issue: Drag image size depends on the initial position and may sometimes
@@ -73,7 +73,7 @@ public class mxSelectionCellsHandler implements MouseListener,
 	/**
 	 * Maps from cells to handlers in the order of the selection cells.
 	 */
-	protected transient Map<Object, mxCellHandler> handlers = new LinkedHashMap<Object, mxCellHandler>();
+	protected transient LinkedHashMap<Object, mxCellHandler> handlers = new LinkedHashMap<Object, mxCellHandler>();
 
 	/**
 	 * 
@@ -82,7 +82,10 @@ public class mxSelectionCellsHandler implements MouseListener,
 	{
 		public void invoke(Object source, mxEventObject evt)
 		{
-			refresh();
+			if (isEnabled())
+			{
+				refresh();
+			}
 		}
 	};
 
@@ -365,7 +368,7 @@ public class mxSelectionCellsHandler implements MouseListener,
 
 		// Creates a new map for the handlers and tries to
 		// to reuse existing handlers from the old map
-		Map<Object, mxCellHandler> oldHandlers = handlers;
+		LinkedHashMap<Object, mxCellHandler> oldHandlers = handlers;
 		handlers = new LinkedHashMap<Object, mxCellHandler>();
 
 		// Creates handles for all selection cells
@@ -377,9 +380,9 @@ public class mxSelectionCellsHandler implements MouseListener,
 		{
 			mxCellState state = graph.getView().getState(tmp[i]);
 
-			if (state != null)
+			if (state != null && state.getCell() != graph.getView().getCurrentRoot())
 			{
-				mxCellHandler handler = oldHandlers.get(tmp[i]);
+				mxCellHandler handler = oldHandlers.remove(tmp[i]);
 
 				if (handler != null)
 				{
@@ -413,6 +416,11 @@ public class mxSelectionCellsHandler implements MouseListener,
 				}
 			}
 		}
+		
+		for (mxCellHandler handler: oldHandlers.values())
+		{
+			handler.destroy();
+		}
 
 		Rectangle dirty = bounds;
 
