@@ -1,21 +1,9 @@
 /**
- * $Id: mxConstants.java,v 1.71 2011-07-18 15:28:02 david Exp $
- * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
+ * $Id: mxConstants.java,v 1.80 2012-05-24 06:39:54 gaudenz Exp $
+ * Copyright (c) 2007-2012, JGraph Ltd
  */
 package com.mxgraph.util;
 
-import java.awt.BasicStroke;
-import java.awt.Color;
-import java.awt.Component;
-import java.awt.Graphics;
-import java.awt.Graphics2D;
-import java.awt.Image;
-import java.awt.Stroke;
-import java.awt.image.BufferedImage;
-
-import javax.swing.border.Border;
-import javax.swing.border.LineBorder;
-
 /**
  * Contains all global constants.
  */
@@ -31,6 +19,12 @@ public class mxConstants
 	 */
 	public static double DEG_PER_RAD = 57.2957795;
 
+	/**
+	 * Defines the ratio between px units in HTML markup and pixels on screen.
+	 * Default is 0.77.
+	 */
+	public static double PX_PER_PIXEL = 0.77;
+
 	/**
 	 * Defines the minimum scale at which rounded polylines should be painted.
 	 * Default is 0.05.
@@ -70,26 +64,6 @@ public class mxConstants
 	 */
 	public static String NS_XLINK = "http://www.w3.org/1999/xlink";
 
-	/**
-	 * Contains an empty image of size 1, 1.
-	 */
-	public static BufferedImage EMPTY_IMAGE;
-
-	static
-	{
-		try
-		{
-			mxConstants.EMPTY_IMAGE = new BufferedImage(1, 1,
-					BufferedImage.TYPE_INT_RGB);
-		}
-		catch (Exception e)
-		{
-			// Occurs when running on GAE, BufferedImage is a
-			// blacklisted class
-			mxConstants.EMPTY_IMAGE = null;
-		}
-	}
-	
 	/**
 	 * Comma separated list of default fonts for CSS properties.
 	 * Default is Arial, Helvetica.
@@ -117,6 +91,11 @@ public class mxConstants
 	 */
 	public static int LINESPACING = 0;
 
+	/**
+	 * Whether or not to split whole words when applying word wrapping in mxUtils.wordWrap.
+	 */
+	public static boolean SPLIT_WORDS = true;
+
 	/**
 	 * Defines the inset in absolute pixels between the label bounding box and
 	 * the label text. Default is 3.
@@ -140,9 +119,14 @@ public class mxConstants
 	public static int DEFAULT_IMAGESIZE = 24;
 
 	/**
-	 * Defines the color to be used for shadows. Default is gray.
+	 * Defines the default opacity for stencils shadows. Default is 1.
 	 */
-	public static Color SHADOW_COLOR = Color.gray;
+	public static int STENCIL_SHADOW_OPACITY = 1;
+
+	/**
+	 * Defines the default shadow color for stencils. Default is "gray".
+	 */
+	public static String STENCIL_SHADOWCOLOR = "gray";
 
 	/**
 	 * Defines the x-offset to be used for shadows. Default is 2.
@@ -165,16 +149,6 @@ public class mxConstants
 	 */
 	public static String SVG_SHADOWTRANSFORM = "translate(2 3)";
 
-	/**
-	 * Specifies the default valid color. Default is green.
-	 */
-	public static Color DEFAULT_VALID_COLOR = Color.GREEN;
-
-	/**
-	 * Specifies the default invalid color. Default is red.
-	 */
-	public static Color DEFAULT_INVALID_COLOR = Color.RED;
-
 	/**
 	 * Specifies the default dash pattern, 3 pixels solid, 3 pixels clear.
 	 */
@@ -186,16 +160,6 @@ public class mxConstants
 	 */
 	public static double DEFAULT_LABEL_BUFFER = 12.0;
 
-	/**
-	 * Defines the rubberband border color. 
-	 */
-	public static Color RUBBERBAND_BORDERCOLOR = new Color(51, 153, 255);
-
-	/**
-	 * Defines the rubberband fill color with an alpha of 80.
-	 */
-	public static Color RUBBERBAND_FILLCOLOR = new Color(51, 153, 255, 80);
-
 	/**
 	 * Defines the handle size. Default is 7.
 	 */
@@ -206,31 +170,6 @@ public class mxConstants
 	 */
 	public static int LABEL_HANDLE_SIZE = 4;
 
-	/**
-	 * Defines the handle border color. Default is black.
-	 */
-	public static Color HANDLE_BORDERCOLOR = Color.black;
-
-	/**
-	 * Defines the handle fill color. Default is green.
-	 */
-	public static Color HANDLE_FILLCOLOR = Color.green;
-
-	/**
-	 * Defines the label handle fill color. Default is yellow.
-	 */
-	public static Color LABEL_HANDLE_FILLCOLOR = Color.yellow;
-
-	/**
-	 * Defines the connect handle fill color. Default is blue.
-	 */
-	public static Color CONNECT_HANDLE_FILLCOLOR = Color.blue;
-
-	/**
-	 * Defines the handle fill color for locked handles. Default is red.
-	 */
-	public static Color LOCKED_HANDLE_FILLCOLOR = Color.red;
-
 	/**
 	 * Defines the default value for the connect handle. Default is false.
 	 */
@@ -241,60 +180,6 @@ public class mxConstants
 	 */
 	public static int CONNECT_HANDLE_SIZE = 8;
 
-	/**
-	 * Defines the selection color for edges. Default is green.
-	 */
-	public static Color EDGE_SELECTION_COLOR = Color.green;
-
-	/**
-	 * Defines the selection color for vertices. Default is green.
-	 */
-	public static Color VERTEX_SELECTION_COLOR = Color.green;
-
-	/**
-	 * Defines the stroke used for painting selected edges. Default is a dashed
-	 * line.
-	 */
-	public static Stroke EDGE_SELECTION_STROKE = new BasicStroke(1,
-			BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, new float[] {
-					3, 3 }, 0.0f);
-
-	/**
-	 * Defines the stroke used for painting the border of selected vertices.
-	 * Default is a dashed line.
-	 */
-	public static Stroke VERTEX_SELECTION_STROKE = new BasicStroke(1,
-			BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, new float[] {
-					3, 3 }, 0.0f);
-
-	/**
-	 * Defines the stroke used for painting the preview for new and existing edges
-	 * that are being changed. Default is a dashed line.
-	 */
-	public static Stroke PREVIEW_STROKE = new BasicStroke(1,
-			BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, new float[] {
-					3, 3 }, 0.0f);
-
-	/**
-	 * Defines the border used for painting the preview when vertices are being
-	 * resized, or cells and labels are being moved.
-	 */
-	public static Border PREVIEW_BORDER = new LineBorder(
-			mxConstants.HANDLE_BORDERCOLOR)
-	{
-		/**
-		 * 
-		 */
-		private static final long serialVersionUID = 1348016511717964310L;
-
-		public void paintBorder(Component c, Graphics g, int x, int y,
-				int width, int height)
-		{
-			((Graphics2D) g).setStroke(VERTEX_SELECTION_STROKE);
-			super.paintBorder(c, g, x, y, width, height);
-		}
-	};
-
 	/**
 	 * Defines the length of the horizontal segment of an Entity Relation.
 	 * This can be overridden using mxConstants.STYLE_SEGMENT style.
@@ -642,6 +527,18 @@ public class mxConstants
 	 */
 	public static String STYLE_IMAGE_FLIPV = "imageFlipV";
 
+	/**
+	 * Defines the key for the horizontal stencil flip. This style is only used
+	 * for <mxStencilShape>. Possible values are 0 and 1. Default is 0.
+	 */
+	public static String STYLE_STENCIL_FLIPH = "stencilFlipH";
+
+	/**
+	 * Defines the key for the vertical stencil flip. This style is only used
+	 * for <mxStencilShape>. Possible values are 0 and 1. Default is 0.
+	 */
+	public static String STYLE_STENCIL_FLIPV = "stencilFlipV";
+
 	/**
 	 * Defines the key for the noLabel style. If this is
 	 * true then no label is visible for a given cell.
@@ -766,6 +663,18 @@ public class mxConstants
 	 */
 	public static String STYLE_STARTSIZE = "startSize";
 
+	/**
+	 * Defines the key for the endFill style. Use 0 for no fill or 1
+	 * (default) for fill. (This style is only exported via <mxImageExport>.)
+	 */
+	public static String STYLE_ENDFILL = "endFill";
+
+	/**
+	 * Defines the key for the startFill style. Use 0 for no fill or 1
+	 * (default) for fill. (This style is only exported via <mxImageExport>.)
+	 */
+	public static String STYLE_STARTFILL = "startFill";
+
 	/**
 	 * Defines the key for the dashed style. The type of this value is
 	 * <code>boolean</code> and the value determines whether or not an edge or
@@ -776,7 +685,12 @@ public class mxConstants
 	/**
 	 * Defines the key for the dashed pattern style. The type of this value
 	 * is <code>float[]</code> and the value specifies the dashed pattern 
-	 * to apply to edges drawn with this style.
+	 * to apply to edges drawn with this style. This style allows the user
+	 * to specify a custom-defined dash pattern. This is done using a series
+	 * of numbers. Dash styles are defined in terms of the length of the dash
+	 * (the drawn part of the stroke) and the length of the space between the
+	 * dashes. The lengths are relative to the line width: a length of "1" is
+	 * equal to the line width.
 	 */
 	public static String STYLE_DASH_PATTERN = "dashPattern";
 
