@@ -1,5 +1,5 @@
 /**
- * $Id: mxEvent.java,v 1.12 2010-06-11 10:08:02 gaudenz Exp $
+ * $Id: mxEvent.java,v 1.14 2011-01-28 08:40:17 gaudenz Exp $
  * Copyright (c) 2007, Gaudenz Alder
  */
 package com.mxgraph.util;
@@ -35,6 +35,11 @@ public class mxEvent
 	 */
 	public static final String CONNECT_CELL = "connectCell";
 
+	/**
+	 * 
+	 */
+	public static final String CONNECT = "connect";
+
 	/**
 	 * 
 	 */
@@ -262,6 +267,11 @@ public class mxEvent
 	 */
 	public static final String CLEAR = "clear";
 
+	/**
+	 * 
+	 */
+	public static final String FIRED = "fired";
+
 	/**
 	 * 
 	 */
