@@ -1,5 +1,5 @@
 /**
- * $Id: mxEventSource.java,v 1.11 2010-01-29 10:16:30 gaudenz Exp $
+ * $Id: mxEventSource.java,v 1.12 2011-08-26 06:45:18 gaudenz Exp $
  * Copyright (c) 2007, Gaudenz Alder
  */
 package com.mxgraph.util;
@@ -128,7 +128,7 @@ public class mxEventSource
 	{
 		if (eventListeners != null)
 		{
-			for (int i = eventListeners.size() - 2; i > 1; i -= 2)
+			for (int i = eventListeners.size() - 2; i > -1; i -= 2)
 			{
 				if (eventListeners.get(i + 1) == listener
 						&& (eventName == null || String.valueOf(
