@@ -1,6 +1,6 @@
 /**
- * $Id: mxUtils.java,v 1.113 2011-07-25 20:45:46 gaudenz Exp $
- * Copyright (c) 2007-2011, Gaudenz Alder, David Benson
+ * $Id: mxUtils.java,v 1.129 2012-08-02 10:58:37 gaudenz Exp $
+ * Copyright (c) 2007-2012, JGraph Ltd
  */
 package com.mxgraph.util;
 
@@ -23,9 +23,8 @@ import java.io.ByteArrayInputStream;
 import java.io.FileInputStream;
 import java.io.FileWriter;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.InputStreamReader;
-import java.io.StringReader;
-import java.io.StringWriter;
 import java.net.URL;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
@@ -45,11 +44,6 @@ import javax.imageio.ImageIO;
 import javax.swing.text.html.HTMLDocument;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
-import javax.xml.transform.OutputKeys;
-import javax.xml.transform.Transformer;
-import javax.xml.transform.TransformerFactory;
-import javax.xml.transform.dom.DOMSource;
-import javax.xml.transform.stream.StreamResult;
 import javax.xml.xpath.XPath;
 import javax.xml.xpath.XPathConstants;
 import javax.xml.xpath.XPathExpressionException;
@@ -59,7 +53,6 @@ import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
-import org.xml.sax.InputSource;
 
 import com.mxgraph.io.mxCodecRegistry;
 import com.mxgraph.model.mxCellPath;
@@ -106,12 +99,25 @@ public class mxUtils
 	 */
 	public static mxRectangle getLabelSize(String label,
 			Map<String, Object> style, boolean isHtml, double scale)
+	{
+		return getLabelSize(label, style, isHtml, scale, 0);
+	}
+
+	/**
+	 * Returns the size for the given label. If isHtml is true then any HTML
+	 * markup in the label is computed as HTML and all newlines inside the HTML
+	 * body are converted into linebreaks.
+	 */
+	public static mxRectangle getLabelSize(String label,
+			Map<String, Object> style, boolean isHtml, double scale,
+			double htmlWrapWidth)
 	{
 		mxRectangle size;
 
 		if (isHtml)
 		{
-			size = getSizeForHtml(getBodyMarkup(label, true), style, scale);
+			size = getSizeForHtml(getBodyMarkup(label, true), style, scale,
+					htmlWrapWidth);
 		}
 		else
 		{
@@ -155,7 +161,18 @@ public class mxUtils
 			Map<String, Object> style, boolean isHtml, mxPoint offset,
 			mxRectangle vertexBounds, double scale)
 	{
-		mxRectangle size = mxUtils.getLabelSize(label, style, isHtml, scale);
+		double wrapWidth = 0;
+
+		if (isHtml
+				&& vertexBounds != null
+				&& mxUtils.getString(style, mxConstants.STYLE_WHITE_SPACE,
+						"nowrap").equals("wrap"))
+		{
+			wrapWidth = vertexBounds.getWidth();
+		}
+
+		mxRectangle size = mxUtils.getLabelSize(label, style, isHtml, scale,
+				wrapWidth);
 
 		// Measures font with full scale and scales back
 		size.setWidth(size.getWidth() / scale);
@@ -340,19 +357,30 @@ public class mxUtils
 
 		Rectangle2D boundingBox = null;
 
-		for (int i = 0; i < lines.length; i++)
+		if (lines.length == 0)
 		{
-			Rectangle2D bounds = font.getStringBounds(lines[i], frc);
+			boundingBox = font.getStringBounds("", frc);
+		}
+		else
+		{
+			for (int i = 0; i < lines.length; i++)
+			{
+				Rectangle2D bounds = font.getStringBounds(lines[i], frc);
 
-			if (boundingBox == null)
-			{
-				boundingBox = bounds;
-			}
-			else
-			{
-				boundingBox.setFrame(0, 0,
-						Math.max(boundingBox.getWidth(), bounds.getWidth()),
-						boundingBox.getHeight() + lineHeight);
+				if (boundingBox == null)
+				{
+					boundingBox = bounds;
+				}
+				else
+				{
+					boundingBox
+							.setFrame(
+									0,
+									0,
+									Math.max(boundingBox.getWidth(),
+											bounds.getWidth()),
+									boundingBox.getHeight() + lineHeight);
+				}
 			}
 		}
 
@@ -449,7 +477,7 @@ public class mxUtils
 						wordStack.push(word.trim());
 						lineWidth = 0;
 					}
-					else
+					else if (mxConstants.SPLIT_WORDS)
 					{
 						// There are no words on the current line and the 
 						// current word does not fit on it. Find the maximum
@@ -486,6 +514,18 @@ public class mxUtils
 							}
 						}
 					}
+					else
+					{
+						// There are no words on the current line, but
+						// we are not splitting.
+						word = word.trim();
+						result.add(word);
+						currentLine = new StringBuilder();
+						lineWidth = 0;
+						// Increment char counter allowing for white 
+						// space in the original word
+						charCount = word.length() + whitespaceCount;
+					}
 				}
 				else
 				{
@@ -507,12 +547,7 @@ public class mxUtils
 				}
 			}
 
-			// If there is anything in the current line after processing all of
-			// the words in this line, add it to the result.
-			if (currentLine.length() > 0 || result.isEmpty())
-			{
-				result.add(currentLine.toString());
-			}
+			result.add(currentLine.toString());
 		}
 
 		return result.toArray(new String[result.size()]);
@@ -526,16 +561,38 @@ public class mxUtils
 	 *            HTML markup whose size should be returned.
 	 */
 	public static mxRectangle getSizeForHtml(String markup,
-			Map<String, Object> style, double scale)
+			Map<String, Object> style, double scale, double wrapWidth)
 	{
 		mxLightweightLabel textRenderer = mxLightweightLabel
 				.getSharedInstance();
 
 		if (textRenderer != null)
 		{
+			// First run measures size with no wrapping
 			textRenderer.setText(createHtmlDocument(style, markup));
 			Dimension size = textRenderer.getPreferredSize();
 
+			// Second run measures size with wrapping if required.
+			// Note that this is only required because max-width
+			// is not supported and we can't get the width of an
+			// inner HTML element (or is this possible?).
+			if (wrapWidth > 0)
+			{
+				textRenderer.setText(createHtmlDocument(
+						style,
+						markup,
+						1,
+						(int) Math.ceil(wrapWidth * mxConstants.PX_PER_PIXEL
+								- mxConstants.LABEL_INSET * scale)));
+				Dimension size2 = textRenderer.getPreferredSize();
+
+				// Uses wrapped text size if any text was actually wrapped
+				if (size2.width < size.width)
+				{
+					size = size2;
+				}
+			}
+
 			return new mxRectangle(0, 0, size.width * scale, size.height
 					* scale);
 		}
@@ -545,6 +602,125 @@ public class mxUtils
 		}
 	}
 
+	/**
+	 * Function: arcToCurves
+	 * 
+	 * Converts the given arc to a series of curves.
+	 */
+	public static double[] arcToCurves(double x0, double y0, double r1,
+			double r2, double angle, double largeArcFlag, double sweepFlag,
+			double x, double y)
+	{
+		x -= x0;
+		y -= y0;
+
+		if (r1 == 0 || r2 == 0)
+		{
+			return new double[0];
+		}
+
+		double fS = sweepFlag;
+		double psai = angle;
+		r1 = Math.abs(r1);
+		r2 = Math.abs(r2);
+		double ctx = -x / 2;
+		double cty = -y / 2;
+		double cpsi = Math.cos(psai * Math.PI / 180);
+		double spsi = Math.sin(psai * Math.PI / 180);
+		double rxd = cpsi * ctx + spsi * cty;
+		double ryd = -1 * spsi * ctx + cpsi * cty;
+		double rxdd = rxd * rxd;
+		double rydd = ryd * ryd;
+		double r1x = r1 * r1;
+		double r2y = r2 * r2;
+		double lamda = rxdd / r1x + rydd / r2y;
+		double sds;
+
+		if (lamda > 1)
+		{
+			r1 = Math.sqrt(lamda) * r1;
+			r2 = Math.sqrt(lamda) * r2;
+			sds = 0;
+		}
+		else
+		{
+			double seif = 1;
+
+			if (largeArcFlag == fS)
+			{
+				seif = -1;
+			}
+
+			sds = seif
+					* Math.sqrt((r1x * r2y - r1x * rydd - r2y * rxdd)
+							/ (r1x * rydd + r2y * rxdd));
+		}
+
+		double txd = sds * r1 * ryd / r2;
+		double tyd = -1 * sds * r2 * rxd / r1;
+		double tx = cpsi * txd - spsi * tyd + x / 2;
+		double ty = spsi * txd + cpsi * tyd + y / 2;
+		double rad = Math.atan2((ryd - tyd) / r2, (rxd - txd) / r1)
+				- Math.atan2(0, 1);
+		double s1 = (rad >= 0) ? rad : 2 * Math.PI + rad;
+		rad = Math.atan2((-ryd - tyd) / r2, (-rxd - txd) / r1)
+				- Math.atan2((ryd - tyd) / r2, (rxd - txd) / r1);
+		double dr = (rad >= 0) ? rad : 2 * Math.PI + rad;
+
+		if (fS == 0 && dr > 0)
+		{
+			dr -= 2 * Math.PI;
+		}
+		else if (fS != 0 && dr < 0)
+		{
+			dr += 2 * Math.PI;
+		}
+
+		double sse = dr * 2 / Math.PI;
+		int seg = (int) Math.ceil(sse < 0 ? -1 * sse : sse);
+		double segr = dr / seg;
+		double t = 8 / 3 * Math.sin(segr / 4) * Math.sin(segr / 4)
+				/ Math.sin(segr / 2);
+		double cpsir1 = cpsi * r1;
+		double cpsir2 = cpsi * r2;
+		double spsir1 = spsi * r1;
+		double spsir2 = spsi * r2;
+		double mc = Math.cos(s1);
+		double ms = Math.sin(s1);
+		double x2 = -t * (cpsir1 * ms + spsir2 * mc);
+		double y2 = -t * (spsir1 * ms - cpsir2 * mc);
+		double x3 = 0;
+		double y3 = 0;
+
+		double[] result = new double[seg * 6];
+
+		for (int n = 0; n < seg; ++n)
+		{
+			s1 += segr;
+			mc = Math.cos(s1);
+			ms = Math.sin(s1);
+
+			x3 = cpsir1 * mc - spsir2 * ms + tx;
+			y3 = spsir1 * mc + cpsir2 * ms + ty;
+			double dx = -t * (cpsir1 * ms + spsir2 * mc);
+			double dy = -t * (spsir1 * ms - cpsir2 * mc);
+
+			// CurveTo updates x0, y0 so need to restore it
+			int index = n * 6;
+			result[index] = x2 + x0;
+			result[index + 1] = y2 + y0;
+			result[index + 2] = x3 - dx + x0;
+			result[index + 3] = y3 - dy + y0;
+			result[index + 4] = x3 + x0;
+			result[index + 5] = y3 + y0;
+
+			x2 = x3 + dx;
+			y2 = y3 + dy;
+		}
+
+		return result;
+	}
+
 	/**
 	 * Returns the bounding box for the rotated rectangle.
 	 */
@@ -594,7 +770,8 @@ public class mxUtils
 	 * @param text the string to test for the presence of the input character
 	 * @param inputChar the test character
 	 * @param fromIndex the index position of the string to start from
-	 * @return 
+	 * @return the position of the first character matching the input character
+	 * 			in the given string where the character has no letter preceding it.
 	 */
 	public static int firstCharAt(String text, int inputChar, int fromIndex)
 	{
@@ -693,7 +870,11 @@ public class mxUtils
 
 	/**
 	 * Returns an integer mask of the port constraints of the given map
-	 * @param dict the style map to determine the port constraints for
+	 * @param terminal the cached cell state of the cell to determine the
+	 * 			port constraints for
+	 * @param edge the edge connected to the constrained terminal
+	 * @param source whether or not the edge specified is connected to the
+	 * 			terminal specified at its source end
 	 * @return the mask of port constraint directions
 	 */
 	public static int getPortConstraints(mxCellState terminal,
@@ -705,7 +886,11 @@ public class mxUtils
 
 	/**
 	 * Returns an integer mask of the port constraints of the given map
-	 * @param dict the style map to determine the port constraints for
+	 * @param terminal the cached cell state of the cell to determine the
+	 * 			port constraints for
+	 * @param edge the edge connected to the constrained terminal
+	 * @param source whether or not the edge specified is connected to the
+	 * 			terminal specified at its source end
 	 * @param defaultValue Default value to return if the key is undefined.
 	 * @return the mask of port constraint directions
 	 */
@@ -950,21 +1135,11 @@ public class mxUtils
 	 * @param style
 	 *            String of the form stylename[;key=value].
 	 * @return Returns the stylename from the given formatted string.
+	 * @deprecated Use <code>mxStyleUtils.getStylename(String)</code> (Jan 2012)
 	 */
 	public static String getStylename(String style)
 	{
-		if (style != null)
-		{
-			String[] pairs = style.split(";");
-			String stylename = pairs[0];
-
-			if (stylename.indexOf("=") < 0)
-			{
-				return stylename;
-			}
-		}
-
-		return "";
+		return mxStyleUtils.getStylename(style);
 	}
 
 	/**
@@ -974,125 +1149,32 @@ public class mxUtils
 	 * @param style
 	 *            String of the form stylename[;stylename][;key=value].
 	 * @return Returns the stylename from the given formatted string.
+	 * @deprecated Use <code>mxStyleUtils.getStylenames(String)</code> (Jan 2012)
 	 */
 	public static String[] getStylenames(String style)
 	{
-		List<String> result = new ArrayList<String>();
-
-		if (style != null)
-		{
-			String[] pairs = style.split(";");
-
-			for (int i = 0; i < pairs.length; i++)
-			{
-				if (pairs[i].indexOf("=") < 0)
-				{
-					result.add(pairs[i]);
-				}
-			}
-		}
-
-		return result.toArray(new String[result.size()]);
+		return mxStyleUtils.getStylenames(style);
 	}
 
 	/**
 	 * Returns the index of the given stylename in the given style. This returns
 	 * -1 if the given stylename does not occur (as a stylename) in the given
 	 * style, otherwise it returns the index of the first character.
+	 * @deprecated Use <code>mxStyleUtils.indexOfStylename(String, String)</code> (Jan 2012)
 	 */
 	public static int indexOfStylename(String style, String stylename)
 	{
-		if (style != null && stylename != null)
-		{
-			String[] tokens = style.split(";");
-			int pos = 0;
-
-			for (int i = 0; i < tokens.length; i++)
-			{
-				if (tokens[i].equals(stylename))
-				{
-					return pos;
-				}
-
-				pos += tokens[i].length() + 1;
-			}
-		}
-
-		return -1;
-	}
-
-	/**
-	 * Adds the specified stylename to the given style if it does not already
-	 * contain the stylename.
-	 */
-	public String addStylename(String style, String stylename)
-	{
-		if (indexOfStylename(style, stylename) < 0)
-		{
-			if (style == null)
-			{
-				style = "";
-			}
-			else if (style.length() > 0
-					&& style.charAt(style.length() - 1) != ';')
-			{
-				style += ';';
-			}
-
-			style += stylename;
-		}
-
-		return style;
-	}
-
-	/**
-	 * Removes all occurrences of the specified stylename in the given style and
-	 * returns the updated style. Trailing semicolons are preserved.
-	 */
-	public String removeStylename(String style, String stylename)
-	{
-		StringBuffer buffer = new StringBuffer();
-
-		if (style != null)
-		{
-			String[] tokens = style.split(";");
-
-			for (int i = 0; i < tokens.length; i++)
-			{
-				if (!tokens[i].equals(stylename))
-				{
-					buffer.append(tokens[i] + ";");
-				}
-			}
-		}
-
-		return (buffer.length() > 1) ? buffer.substring(0, buffer.length() - 1)
-				: buffer.toString();
+		return mxStyleUtils.indexOfStylename(style, stylename);
 	}
 
 	/**
 	 * Removes all stylenames from the given style and returns the updated
 	 * style.
+	 * @deprecated Use <code>mxStyleUtils.removeAllStylenames(String)</code> (Jan 2012)
 	 */
 	public static String removeAllStylenames(String style)
 	{
-		StringBuffer buffer = new StringBuffer();
-
-		if (style != null)
-		{
-			String[] tokens = style.split(";");
-
-			for (int i = 0; i < tokens.length; i++)
-			{
-				if (tokens[i].indexOf('=') >= 0)
-				{
-					buffer.append(tokens[i] + ";");
-				}
-			}
-		}
-
-		return (buffer.length() > 1) ? buffer.substring(0, buffer.length() - 1)
-				: buffer.toString();
+		return mxStyleUtils.removeAllStylenames(style);
 	}
 
 	/**
@@ -1107,30 +1189,12 @@ public class mxUtils
 	 *            Key of the style to be changed.
 	 * @param value
 	 *            New value for the given key.
+	 * @deprecated Use <code>mxStyleUtils.setCellStyles(mxIGraphModel, Object[], String, String)</code> (Jan 2012)
 	 */
 	public static void setCellStyles(mxIGraphModel model, Object[] cells,
 			String key, String value)
 	{
-		if (cells != null && cells.length > 0)
-		{
-			model.beginUpdate();
-			try
-			{
-				for (int i = 0; i < cells.length; i++)
-				{
-					if (cells[i] != null)
-					{
-						String style = setStyle(model.getStyle(cells[i]), key,
-								value);
-						model.setStyle(cells[i], style);
-					}
-				}
-			}
-			finally
-			{
-				model.endUpdate();
-			}
-		}
+		mxStyleUtils.setCellStyles(model, cells, key, value);
 	}
 
 	/**
@@ -1145,46 +1209,11 @@ public class mxUtils
 	 * @param value
 	 *            New value for the given key.
 	 * @return Returns the new style.
+	 * @deprecated Use <code>mxStyleUtils.setStyle(String, String, String)</code> (Jan 2012)
 	 */
 	public static String setStyle(String style, String key, String value)
 	{
-		boolean isValue = value != null && value.length() > 0;
-
-		if (style == null || style.length() == 0)
-		{
-			if (isValue)
-			{
-				style = key + "=" + value;
-			}
-		}
-		else
-		{
-			int index = style.indexOf(key + "=");
-
-			if (index < 0)
-			{
-				if (isValue)
-				{
-					String sep = (style.endsWith(";")) ? "" : ";";
-					style = style + sep + key + '=' + value;
-				}
-			}
-			else
-			{
-				String tmp = (isValue) ? key + "=" + value : "";
-				int cont = style.indexOf(";", index);
-
-				if (!isValue)
-				{
-					cont++;
-				}
-
-				style = style.substring(0, index) + tmp
-						+ ((cont > index) ? style.substring(cont) : "");
-			}
-		}
-
-		return style;
+		return mxStyleUtils.setStyle(style, key, value);
 	}
 
 	/**
@@ -1210,30 +1239,12 @@ public class mxUtils
 	 *            Integer for the bit to be changed.
 	 * @param value
 	 *            Optional boolean value for the flag.
+	 * @deprecated Use <code>mxStyleUtils.setCellStyleFlags(mxIGraphModel, Object[],String, int, Boolean)</code> (Jan 2012)
 	 */
 	public static void setCellStyleFlags(mxIGraphModel model, Object[] cells,
 			String key, int flag, Boolean value)
 	{
-		if (cells != null && cells.length > 0)
-		{
-			model.beginUpdate();
-			try
-			{
-				for (int i = 0; i < cells.length; i++)
-				{
-					if (cells[i] != null)
-					{
-						String style = setStyleFlag(model.getStyle(cells[i]),
-								key, flag, value);
-						model.setStyle(cells[i], style);
-					}
-				}
-			}
-			finally
-			{
-				model.endUpdate();
-			}
-		}
+		mxStyleUtils.setCellStyleFlags(model, cells, key, flag, value);
 	}
 
 	/**
@@ -1248,72 +1259,12 @@ public class mxUtils
 	 *            Integer for the bit to be changed.
 	 * @param value
 	 *            Optional boolean value for the given flag.
+	 * @deprecated Use <code>mxStyleUtils.setStyleFlag(String, String, int, Boolean)</code> (Jan 2012)
 	 */
 	public static String setStyleFlag(String style, String key, int flag,
 			Boolean value)
 	{
-		if (style == null || style.length() == 0)
-		{
-			if (value == null || value.booleanValue())
-			{
-				style = key + "=" + flag;
-			}
-			else
-			{
-				style = key + "=0";
-			}
-		}
-		else
-		{
-			int index = style.indexOf(key + "=");
-
-			if (index < 0)
-			{
-				String sep = (style.endsWith(";")) ? "" : ";";
-
-				if (value == null || value.booleanValue())
-				{
-					style = style + sep + key + "=" + flag;
-				}
-				else
-				{
-					style = style + sep + key + "=0";
-				}
-			}
-			else
-			{
-				int cont = style.indexOf(";", index);
-				String tmp = "";
-				int result = 0;
-
-				if (cont < 0)
-				{
-					tmp = style.substring(index + key.length() + 1);
-				}
-				else
-				{
-					tmp = style.substring(index + key.length() + 1, cont);
-				}
-
-				if (value == null)
-				{
-					result = Integer.parseInt(tmp) ^ flag;
-				}
-				else if (value.booleanValue())
-				{
-					result = Integer.parseInt(tmp) | flag;
-				}
-				else
-				{
-					result = Integer.parseInt(tmp) & ~flag;
-				}
-
-				style = style.substring(0, index) + key + "=" + result
-						+ ((cont >= 0) ? style.substring(cont) : "");
-			}
-		}
-
-		return style;
+		return mxStyleUtils.setStyleFlag(style, key, flag, value);
 	}
 
 	public static boolean intersectsHotspot(mxCellState state, int x, int y,
@@ -1517,6 +1468,24 @@ public class mxUtils
 	 */
 	public static float[] getFloatArray(Map<String, Object> dict, String key,
 			float[] defaultValue)
+	{
+		return getFloatArray(dict, key, defaultValue, ",");
+	}
+
+	/**
+	 * Returns the value for key in dictionary as a float array or the given default
+	 * value if no value is defined for the key.
+	 * 
+	 * @param dict
+	 *            Dictionary that contains the key, value pairs.
+	 * @param key
+	 *            Key whose value should be returned.
+	 * @param defaultValue
+	 *            Default value to return if the key is undefined.
+	 * @return Returns the float array value for key in dict.
+	 */
+	public static float[] getFloatArray(Map<String, Object> dict, String key,
+			float[] defaultValue, String separator)
 	{
 		Object value = dict.get(key);
 
@@ -1526,7 +1495,7 @@ public class mxUtils
 		}
 		else
 		{
-			String[] floatChars = value.toString().split(",");
+			String[] floatChars = value.toString().split(separator);
 			float[] result = new float[floatChars.length];
 
 			for (int i = 0; i < floatChars.length; i++)
@@ -1764,15 +1733,37 @@ public class mxUtils
 		{
 			return null;
 		}
+		else if (colorString.startsWith("#") && colorString.length() == 4)
+		{
+			// Adds support for special short notation of hex colors, eg. #abc=#aabbcc
+			colorString = new String(new char[] { '#', colorString.charAt(1),
+					colorString.charAt(1), colorString.charAt(2),
+					colorString.charAt(2), colorString.charAt(3),
+					colorString.charAt(3) });
+		}
 
-		int value;
+		int value = 0;
 		try
 		{
-			value = (int) Long.parseLong(colorString, 16);
+			String tmp = colorString;
+
+			if (tmp.startsWith("#"))
+			{
+				tmp = tmp.substring(1);
+			}
+
+			value = (int) Long.parseLong(tmp, 16);
 		}
 		catch (NumberFormatException nfe)
 		{
-			value = Long.decode(colorString).intValue();
+			try
+			{
+				value = Long.decode(colorString).intValue();
+			}
+			catch (NumberFormatException e)
+			{
+				// ignores exception and returns black
+			}
 		}
 
 		return new Color(value);
@@ -1801,8 +1792,21 @@ public class mxUtils
 	 */
 	public static String readFile(String filename) throws IOException
 	{
-		BufferedReader reader = new BufferedReader(new InputStreamReader(
-				new FileInputStream(filename)));
+		return readInputStream(new FileInputStream(filename));
+	}
+
+	/**
+	 * Reads the given filename into a string.
+	 * 
+	 * @param filename
+	 *            Name of the file to be read.
+	 * @return Returns a string representing the file contents.
+	 * @throws IOException
+	 */
+	public static String readInputStream(InputStream stream) throws IOException
+	{
+		BufferedReader reader = new BufferedReader(
+				new InputStreamReader(stream));
 		StringBuffer result = new StringBuffer();
 		String tmp = reader.readLine();
 
@@ -1982,7 +1986,7 @@ public class mxUtils
 				int type = (background != null) ? BufferedImage.TYPE_INT_RGB
 						: BufferedImage.TYPE_INT_ARGB;
 				result = new BufferedImage(w, h, type);
-
+				
 				// Clears background
 				if (background != null)
 				{
@@ -2143,97 +2147,38 @@ public class mxUtils
 	 * Returns a new, empty DOM document.
 	 * 
 	 * @return Returns a new DOM document.
+	 * @deprecated Use <code>mxDomUtils.createDocument</code> (Jan 2012)
 	 */
 	public static Document createDocument()
 	{
-		Document result = null;
-
-		try
-		{
-			DocumentBuilderFactory factory = DocumentBuilderFactory
-					.newInstance();
-			DocumentBuilder parser = factory.newDocumentBuilder();
-
-			result = parser.newDocument();
-		}
-		catch (Exception e)
-		{
-			System.out.println(e.getMessage());
-		}
-
-		return result;
+		return mxDomUtils.createDocument();
 	}
 
 	/**
 	 * Creates a new SVG document for the given width and height.
+	 * @deprecated Use <code>mxDomUtils.createSvgDocument(int, int)</code> (Jan 2012)
 	 */
 	public static Document createSvgDocument(int width, int height)
 	{
-		Document document = createDocument();
-		Element root = document.createElement("svg");
-
-		String w = String.valueOf(width);
-		String h = String.valueOf(height);
-
-		root.setAttribute("width", w);
-		root.setAttribute("height", h);
-		root.setAttribute("viewBox", "0 0 " + w + " " + h);
-		root.setAttribute("version", "1.1");
-		root.setAttribute("xmlns", mxConstants.NS_SVG);
-		root.setAttribute("xmlns:xlink", mxConstants.NS_XLINK);
-
-		document.appendChild(root);
-
-		return document;
+		return mxDomUtils.createSvgDocument(width, height);
 	}
 
 	/**
 	 * 
+	 * @deprecated Use <code>mxDomUtils.createVmlDocument</code> (Jan 2012)
 	 */
 	public static Document createVmlDocument()
 	{
-		Document document = createDocument();
-
-		Element root = document.createElement("html");
-		root.setAttribute("xmlns:v", "urn:schemas-microsoft-com:vml");
-		root.setAttribute("xmlns:o", "urn:schemas-microsoft-com:office:office");
-
-		document.appendChild(root);
-
-		Element head = document.createElement("head");
-
-		Element style = document.createElement("style");
-		style.setAttribute("type", "text/css");
-		style.appendChild(document
-				.createTextNode("<!-- v\\:* {behavior: url(#default#VML);} -->"));
-
-		head.appendChild(style);
-		root.appendChild(head);
-
-		Element body = document.createElement("body");
-		root.appendChild(body);
-
-		return document;
+		return mxDomUtils.createVmlDocument();
 	}
 
 	/**
 	 * Returns a document with a HTML node containing a HEAD and BODY node.
+	 * @deprecated Use <code>mxDomUtils.createHtmlDocument</code> (Jan 2012)
 	 */
 	public static Document createHtmlDocument()
 	{
-		Document document = createDocument();
-
-		Element root = document.createElement("html");
-
-		document.appendChild(root);
-
-		Element head = document.createElement("head");
-		root.appendChild(head);
-
-		Element body = document.createElement("body");
-		root.appendChild(body);
-
-		return document;
+		return mxDomUtils.createHtmlDocument();
 	}
 
 	/**
@@ -2244,7 +2189,7 @@ public class mxUtils
 	public static String createHtmlDocument(Map<String, Object> style,
 			String text)
 	{
-		return createHtmlDocument(style, text, 1);
+		return createHtmlDocument(style, text, 1, 0);
 	}
 
 	/**
@@ -2254,6 +2199,33 @@ public class mxUtils
 	 */
 	public static String createHtmlDocument(Map<String, Object> style,
 			String text, double scale)
+	{
+		return createHtmlDocument(style, text, scale, 0);
+	}
+
+	/**
+	 * Returns a new, empty DOM document.
+	 * 
+	 * @return Returns a new DOM document.
+	 */
+	public static String createHtmlDocument(Map<String, Object> style,
+			String text, double scale, int width)
+	{
+		return createHtmlDocument(style, text, scale, width, null);
+	}
+
+	/**
+	 * Returns a new, empty DOM document. The head argument can be used to
+	 * provide an optional HEAD section without the HEAD tags as follows:
+	 * 
+	 * <pre>
+	 * mxUtils.createHtmlDocument(style,  text, 1, 0, "<style type=\"text/css\">.classname { color:red; }</style>")
+	 * </pre>
+	 * 
+	 * @return Returns a new DOM document.
+	 */
+	public static String createHtmlDocument(Map<String, Object> style,
+			String text, double scale, int width, String head)
 	{
 		StringBuffer css = new StringBuffer();
 		css.append("font-family:"
@@ -2299,7 +2271,20 @@ public class mxUtils
 			css.append("text-align:right;");
 		}
 
-		return "<html><body style=\"" + css.toString() + "\">" + text
+		if (width > 0)
+		{
+			// LATER: With max-width support, wrapped text can be measured in 1 step
+			css.append("width:" + width + "px;");
+		}
+
+		String result = "<html>";
+
+		if (head != null)
+		{
+			result += "<head>" + head + "</head>";
+		}
+
+		return result + "<body style=\"" + css.toString() + "\">" + text
 				+ "</body></html>";
 	}
 
@@ -2394,36 +2379,11 @@ public class mxUtils
 	 * @param xml
 	 *            String that represents the XML data.
 	 * @return Returns a new XML document.
-	 * @deprecated as of 31.08.2010. Use parseXML(String xml) instead
-	 */
-	public static Document parse(String xml)
-	{
-		return mxUtils.parseXml(xml);
-	}
-
-	/**
-	 * Returns a new document for the given XML string.
-	 * 
-	 * @param xml
-	 *            String that represents the XML data.
-	 * @return Returns a new XML document.
+	 * @deprecated Use <code>mxXmlUtils.parseXml</code> (Jan 2012)
 	 */
 	public static Document parseXml(String xml)
 	{
-		try
-		{
-			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
-					.newInstance();
-			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
-
-			return docBuilder.parse(new InputSource(new StringReader(xml)));
-		}
-		catch (Exception e)
-		{
-			e.printStackTrace();
-		}
-
-		return null;
+		return mxXmlUtils.parseXml(xml);
 	}
 
 	/**
@@ -2530,27 +2490,11 @@ public class mxUtils
 	 * @param node
 	 *            Node to return the XML for.
 	 * @return Returns an XML string.
+	 * @deprecated Use <code>mxXmlUtils.getXml(Node)</code> (Jan 2012)
 	 */
 	public static String getXml(Node node)
 	{
-		try
-		{
-			Transformer tf = TransformerFactory.newInstance().newTransformer();
-
-			tf.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
-			tf.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
-
-			StreamResult dest = new StreamResult(new StringWriter());
-			tf.transform(new DOMSource(node), dest);
-
-			return dest.getWriter().toString();
-		}
-		catch (Exception e)
-		{
-			// ignore
-		}
-
-		return "";
+		return mxXmlUtils.getXml(node);
 	}
 
 	/**
