@@ -1,5 +1,5 @@
 /**
- * $Id: mxGraphView.java,v 1.154 2011-03-23 13:30:21 gaudenz Exp $
+ * $Id: mxGraphView.java,v 1.158 2012-06-20 13:20:09 gaudenz Exp $
  * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.view;
@@ -626,6 +626,7 @@ public class mxGraphView extends mxEventSource
 				{
 					// Updates the points on the source terminal if its an edge
 					mxCellState source = getState(getVisibleTerminal(cell, true));
+					state.setVisibleTerminalState(source, true);
 
 					if (source != null && model.isEdge(source.getCell())
 							&& !model.isAncestor(source, cell))
@@ -638,6 +639,7 @@ public class mxGraphView extends mxEventSource
 					// Updates the points on the source terminal if its an edge
 					mxCellState target = getState(getVisibleTerminal(cell,
 							false));
+					state.setVisibleTerminalState(target, false);
 
 					if (target != null && model.isEdge(target.getCell())
 							&& !model.isAncestor(target, cell))
@@ -741,9 +743,10 @@ public class mxGraphView extends mxEventSource
 		String label = graph.getLabel(state.getCell());
 		Map<String, Object> style = state.getStyle();
 
-		// Applies word wrapping to the label and stores the result in the state
+		// Applies word wrapping to non-HTML labels and stores the result in the state
 		if (label != null
 				&& label.length() > 0
+				&& !graph.isHtmlLabel(state.getCell())
 				&& !graph.getModel().isEdge(state.getCell())
 				&& mxUtils.getString(style, mxConstants.STYLE_WHITE_SPACE,
 						"nowrap").equals("wrap"))
@@ -1385,7 +1388,10 @@ public class mxGraphView extends mxEventSource
 				// if a connected edge has a null-terminal, ie.
 				// source is null and/or target is null and no
 				// additional control points are defined
-				removeState(state.getCell());
+				if (state.getCell() != getCurrentRoot())
+				{
+					removeState(state.getCell());
+				}
 			}
 			else
 			{
@@ -1481,38 +1487,35 @@ public class mxGraphView extends mxEventSource
 				segment = segments[index++];
 			}
 
-			if (segment != 0)
+			double factor = (segment == 0) ? 0 : (dist - length) / segment;
+			mxPoint p0 = state.getAbsolutePoint(index - 1);
+			mxPoint pe = state.getAbsolutePoint(index);
+
+			if (p0 != null && pe != null)
 			{
-				double factor = (dist - length) / segment;
-				mxPoint p0 = state.getAbsolutePoint(index - 1);
-				mxPoint pe = state.getAbsolutePoint(index);
+				double gy = 0;
+				double offsetX = 0;
+				double offsetY = 0;
 
-				if (p0 != null && pe != null)
+				if (geometry != null)
 				{
-					double gy = 0;
-					double offsetX = 0;
-					double offsetY = 0;
+					gy = geometry.getY();
+					mxPoint offset = geometry.getOffset();
 
-					if (geometry != null)
+					if (offset != null)
 					{
-						gy = geometry.getY();
-						mxPoint offset = geometry.getOffset();
-
-						if (offset != null)
-						{
-							offsetX = offset.getX();
-							offsetY = offset.getY();
-						}
+						offsetX = offset.getX();
+						offsetY = offset.getY();
 					}
-
-					double dx = pe.getX() - p0.getX();
-					double dy = pe.getY() - p0.getY();
-					double nx = dy / segment;
-					double ny = dx / segment;
-
-					x = p0.getX() + dx * factor + (nx * gy + offsetX) * scale;
-					y = p0.getY() + dy * factor - (ny * gy - offsetY) * scale;
 				}
+
+				double dx = pe.getX() - p0.getX();
+				double dy = pe.getY() - p0.getY();
+				double nx = (segment == 0) ? 0 : dy / segment;
+				double ny = (segment == 0) ? 0 : dx / segment;
+
+				x = p0.getX() + dx * factor + (nx * gy + offsetX) * scale;
+				y = p0.getY() + dy * factor - (ny * gy - offsetY) * scale;
 			}
 		}
 		else if (geometry != null)
